
/* ------------------------------------------------------------------------
 * project:    scope
 * file:       applicationlog.cpp
 * copyright:  (c) 2023 by <david@larosterna.com>
 * ------------------------------------------------------------------------
 * Display everything written to std::clog
 * ------------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * ------------------------------------------------------------------------ */

#include "applicationlog.h"
#include "util.h"
#include <iostream>
#include <fstream>

using namespace std;

std::stringstream ApplicationLog::s_logstream;

#if defined(NDEBUG)
  bool ApplicationLog::s_filterdebug = true;
#else
  bool ApplicationLog::s_filterdebug = false;
#endif


QStringList ApplicationLog::lines(size_t nskip)
{
  QStringList list;

  size_t idx = 0;
  std::string ln;
  bool debugMode = (not s_filterdebug);
  while (std::getline(s_logstream, ln)) {
    ++idx;
    if (idx > nskip) {
      if (not ln.empty()) {
        if (debugMode or ln.find("[d]") == string::npos)
          list.append( qstr(ln) );
      }
    }
  }

  // the above leaves eofbit set which never gets un-set again
  s_logstream.clear();

  return list;
}

void ApplicationLog::dump(const string &fname)
{
  ofstream os(asPath(fname));
  os << s_logstream.str();
  os.close();
}

void ApplicationLog::reset()
{
  s_logstream.str( std::string() );
}

void ApplicationLog::filterDebugMessages(bool flag)
{
  s_filterdebug = flag;
}


