/* ------------------------------------------------------------------------
 * project:    scope
 * file:       applicationlog.h
 * copyright:  (c) 2023 by <david@larosterna.com>
 * ------------------------------------------------------------------------
 * Display everything written to std::clog
 * ------------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * ------------------------------------------------------------------------ */

#ifndef SCOPE_APPLICATIONLOG_H
#define SCOPE_APPLICATIONLOG_H

#include "forward.h"
#include <QStringList>
#include <QApplication>
#include <sstream>

/** Application-wide log facility
  *
  * In main(), the standard log stream is redirected to this static object,
  * so that everything written to std::clog ends up here.
  *
  * Some parts of the libraries used do not depend on Qt and therefore write
  * messages to the std streams, which can be rdbuf()'ed to pipe into this
  * stream.
  */
class ApplicationLog
{
public:

  /// access log stream
  static std::stringstream &stream() { return s_logstream; }

  /// extract any new lines into a QStringList
  static QStringList lines(size_t nskip = 0);

  /// write entire contents to file
  static void dump(const std::string &fname);

  /// clear out contents
  static void reset();

  /// whether to remove debug messages from log (can be a lot)
  static void filterDebugMessages(bool flag);

private:

  /// stream to use
  static std::stringstream s_logstream;

  /// whether to filter out debug messages
  static bool s_filterdebug;
};

#endif // APPLICATIONLOG_H
