
/*
 * f2c converted NACA 6-series coordinate generation program
 * originally written by
 * 
 * Charles L. Ladson and Cuyler W. Brooks, Jr. NASA Langley Research Center 
 * Liam Hardy, NASA Ames Research Center 
 * Ralph L. Carmichael, Public Domain Aeronautical Software 
 *
 * REFERENCES - NASA Technical Memorandum TM X-3069 (September, 1974) 
 *              by Charles L. Ladson and Cuyler W. Brooks, Jr.
 * 
 * This file is in the public domain.
 */
 
#include <math.h>
#include "naca6.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Table of constant values */

static const int c__201 = 201;
static const int c__251 = 251;

/* file scope static functions */

static int phep63_(double *phi, double *eps);
static int phep64_(double *phi, double *eps);
static int phep65_(double *phi, double *eps);
static int phep66_(double *phi, double *eps);
static int phep67_(double *phi, double *eps);
static int phep63a_(double *phi, double *eps);
static int phep64a_(double *phi, double *eps);
static int phep65a_(double *phi, double *eps);

static int phps63_(double *phi, double *psi);
static int phps64_(double *phi, double *psi);
static int phps65_(double *phi, double *psi);
static int phps66_(double *phi, double *psi);
static int phps67_(double *phi, double *psi);
static int phps63a_(double *phi, double *psi);
static int phps64a_(double *phi, double *psi);
static int phps65a_(double *phi, double *psi);

static int spline_(int n, double *x, double *y, double *b, double *c__, double *d__);
static int seval3_(int n, double *u, double *x, double *y,
		   double *b, double *c__, double *d__, double *seval,
		   double *deriv, double *deriv2, double *deriv3);

int naca6(int iprofile, int icamber, double toc,
          int ncmbl, double *cli, double *a, int *nout, double *xyout)
{
  /* Initialized data */
  static const double chd = 1.;
  static const double dx = .01;
  static const double e = 1e-11;

  /* System generated locals */
  int i__1;
  double d__1, d__2, d__3;

  /* Local variables */
  double frac, axlc[201], aylc[201], clis, ycmb[10];
  int icky;
  double axuc[201], amxl, ayuc[201], aylp[201], aypp[201], ymax, omxl,
    ayup[201], ylpr[200], ytpp[201], yupr[200];
  int if6xa;
  double amxl1, amxl2, omxl1, omxl2, f, g, h__;
  int i__, j, k, l;
  double q, u, v, x, y, acrat, z__;
  double tanth[10], costh, b1, b2, sinth;
  double dummy, s1, s2, x1, x2, y1, y2, z1, z2, ytppp[201];
  double tanth0[10], an, bn, cn, as, sf, xc;
  int it;
  double yc, ax[201], xl[200], yl[200], tr, xo, ym, yp;
  double xt[201], xu[200], yu[200], yt[201];
  double ali, pia, phi[201], xal[200], rat, xlc, ylc, yal[200], eps[201], 
         thp, psi[201], xau[200], yau[200], rnp, xll, xuc, yuc, ayp[201],
         ylp, ypp, xtp, ytp[201], xym, yup, bspline[201], xsv, cspline[201], 
         dspline[201], ycp2[10];

  /* Initialize uninitialized variables to make compiler happy */
  ali = s2 = s1 = b1 = b2 = 0.0;
         
  /* Check arguments for correctness */
  if ((iprofile < 63) || (iprofile > 67 && iprofile < 163) || (iprofile > 165))
    return NACA6_INVALID_FAMILY;

  if ((icamber < 63) || (icamber > 67 && icamber < 163) || (icamber > 165))
    return NACA6_INVALID_CAMBER;

  if (toc < 0.0)
    return NACA6_INVALID_TOC;

  if (ncmbl > 10 || ncmbl < 1)
    return NACA6_TOOMANYLINES;

  if (cli == 0 || a == 0 || nout == 0 || xyout == 0)
    return NACA6_ZERO_POINTER;

  /* Parameter adjustments */
  --xyout;
  --a;
  --cli;

  icky = ncmbl;
  if (icky < 1) {
    icky = 1;
  }
  for (j = icky + 1; j <= 10; ++j) {
    cli[j] = 0.;
    a[j] = 0.;
/* L110: */
  }
  if6xa = 0;
  frac = 1.;

  if (icamber == 0) {
    a[1] = 0.;
    cli[1] = 0.;
    icky = 1;
  }
/*     Slope of camberline at origin, TANTH0: */
  clis = cli[1];
  as = a[1];
  l = 0;

L200:
  ++l;
  a[1] = a[l];
  cli[1] = cli[l];
  x = 0.;
  y = 0.;
  xc = 0.;
  yc = 0.;
  xu[0] = 0.;
  yu[0] = 0.;
  xl[0] = 0.;
  yl[0] = 0.;
  xuc = 0.;
  yuc = 0.;
  xlc = 0.;
  ylc = 0.;
  xau[0] = 0.;
  yau[0] = 0.;
  xal[0] = 0.;
  yal[0] = 0.;
  k = 2;
/* never used ?????? */
  u = .005;
  v = -(a[1] - u) / (d__1 = a[1] - u, fabs(d__1));
  omxl = (1. - u) * log(1. - u);
  amxl = (a[1] - u) * log((d__1 = a[1] - u, fabs(d__1)));
  omxl1 = -log(1. - u) - 1.;
  amxl1 = -log((d__1 = a[1] - u, fabs(d__1))) + v;
  omxl2 = 1. / (1. - u);
  amxl2 = -v / (d__1 = a[1] - u, fabs(d__1));
  if (a[1] >= e && (d__1 = 1. - a[1], fabs(d__1)) >= e) {
    g = -(a[1] * a[1] * (log(a[1]) * .5 - .25) + .25) / (1. - a[1]);
    q = 1.;
/* Computing 2nd power */
    d__1 = 1. - a[1];
/* Computing 2nd power */
    d__2 = 1. - a[1];
    h__ = (d__1 * d__1 * .5 * log(1. - a[1]) - d__2 * d__2 * .25) / (1. - a[1]) + g;
/* Computing 2nd power */
    d__1 = a[1] - u;
/* Computing 2nd power */
    d__2 = 1. - u;
    z__ = (a[1] - u) * .5 * amxl - (1. - u) * .5 * omxl - d__1 * d__1 * .25 + d__2 * d__2 * .25;
    z1 = ((a[1] - u) * amxl1 - amxl - (1. - u) * omxl1 + omxl + (a[1] - u)
	  - (1. - u)) * .5;
    z2 = (a[1] - u) * .5 * amxl2 - amxl1 - (1. - u) * .5 * omxl2 + omxl1;
  }
  if (a[1] < e) {
    h__ = -.5;
    q = 1.;
    z1 = u * log(u) - u * .5 - (1. - u) * .5 * omxl1 + omxl * .5 - .5;
  } else if ((d__1 = a[1] - 1., fabs(d__1)) < e) {
    h__ = 0.;
    q = 0.;
    z1 = -omxl1;
  }
  tanth0[l - 1] = cli[1] * (z1 / (1. - q * a[1]) - 1. - log(u) - h__) /
    3.14159265 / (a[1] + 1.) / 2.;
  if (l < icky && icky > 1) {
    goto L200;
  }
  i__1 = icky;
  for (j = 2; j <= i__1; ++j) {
    tanth0[0] += tanth0[j - 1];
/* L210: */
  }
/* ..... Slope of profile at origin, Upper and Lower: */
  yp = 1e11;
  ypp = 1e11;
  if (tanth0[0] != 0.) {
    yup = -1. / tanth0[0];
    ylp = -1. / tanth0[0];
  } else {
    yup = 0.;
    ylp = 0.;
  }
/*     First station aft of origin on uncambered profile: */
  i__ = 1;
  x = 2.5e-4;
/*     Start loop for X increment: */
L300:
/*       Skip thickness computation after first pass: */
/* keep coming back here */
  if (i__ > 1) {
    goto L550;
  }
/*       Select series: */
  if (iprofile == 63) {
    phep63_(phi, eps);
  }
  if (iprofile == 64) {
    phep64_(phi, eps);
  }
  if (iprofile == 65) {
    phep65_(phi, eps);
  }
  if (iprofile == 66) {
    phep66_(phi, eps);
  }
  if (iprofile == 67) {
    phep67_(phi, eps);
  }
  if (iprofile == 63) {
    phps63_(phi, psi);
  }
  if (iprofile == 64) {
    phps64_(phi, psi);
  }
  if (iprofile == 65) {
    phps65_(phi, psi);
  }
  if (iprofile == 66) {
    phps66_(phi, psi);
  }
  if (iprofile == 67) {
    phps67_(phi, psi);
  }
  if (iprofile == 163) {
    phep63a_(phi, eps);
  }
  if (iprofile == 164) {
    phep64a_(phi, eps);
  }
  if (iprofile == 165) {
    phep65a_(phi, eps);
  }
  if (iprofile == 163) {
    phps63a_(phi, psi);
  }
  if (iprofile == 164) {
    phps64a_(phi, psi);
  }
  if (iprofile == 165) {
    phps65a_(phi, psi);
  }
  rat = 1.;
  it = 0;
  acrat = 1.;
/*       Loop start for thickness iteration: */
L400:
/* was 150 */
  ++it;
  acrat *= rat;
  ymax = 0.;
  for (j = 1; j <= 201; ++j) {
/* changed by RLC from NX  1Dec94 */
    xt[j - 1] = cosh(psi[j - 1] * acrat) * -2. * cos(phi[j - 1] - eps[j - 1] * acrat);
    yt[j - 1] = sinh(psi[j - 1] * acrat) * 2. * sin(phi[j - 1] - eps[j - 1] * acrat);
    if (yt[j - 1] > ymax) {
      xym = xt[j - 1];
    }
    if (yt[j - 1] > ymax) {
      ymax = yt[j - 1];
    }
/* L410: */
  }
/* .... Estimate first,second and third derivatives by spline fit......... */
/* was 160 */
  spline_(c__201, xt, yt, bspline, cspline, dspline);
  for (j = 1; j <= 201; ++j) {
    seval3_(c__201, &xt[j - 1], xt, yt, bspline, cspline, dspline, &dummy, &ytp[j - 1],
	    &ytpp[j - 1], &ytppp[j - 1]);
  }

/* ..... Estimate the location of maximum thickness. Look for the interval */
/*     where dt/dx changes sign. XTP,YM are x,y of max thickness */
  xtp = 1.;
  for (j = 3; j <= 200; ++j) {
    if (ytp[j - 1] < 0. && ytp[j - 2] >= 0.) {
      xtp = xt[j - 2] + ytp[j - 2] * (xt[j - 1] - xt[j - 2]) / (ytp[j - 2] - ytp[j - 1]);
    }
/* L420: */
  }
  seval3_(c__201, &xtp, xt, yt, bspline, cspline, dspline, &ym, &dummy, &dummy, &dummy);
  xo = xt[0];
  xl[0] = xt[199];
  tr = ym * 2. / (xl[0] - xo);
  rat = toc / tr;
  sf = rat;
  if (toc > e && (d__1 = rat - 1., fabs(d__1)) > 1e-4 && it <= 10) {
    goto L400;
  }
/* keep going back until TR=TOC */
  if (i__ == 1) {
    for (j = 1; j <= 201; ++j) {
      xt[j - 1] = (xt[j - 1] - xo) / (xl[0] - xo);
/*           Scale linearly to exact thickness: */
      yt[j - 1] = sf * yt[j - 1] / (xl[0] - xo);
      ytp[j - 1] = sf * ytp[j - 1];
      ytpp[j - 1] = sf * ytpp[j - 1] * (xl[0] - xo);
/* L500: */
    }
  }
/* ..... Now the XT,YT,YTP,YTPP array are loaded and never change ........ */
  xtp = (xtp - xo) / (xl[0] - xo);
  ymax = ymax * sf / (xl[0] - xo);
  ym = ym * sf / (xl[0] - xo);
  xym = (xym - xo) / (xl[0] - xo);
  xl[0] = 0.;
  if (toc > e) {
/*        Fit tilted ellipse at eleventh profile point: */
    cn = ytp[10] * 2. - yt[10] / xt[10] + .1;
    an = xt[10] * (ytp[10] * xt[10] - yt[10]) / (xt[10] * (ytp[10] * 2. - cn) - yt[10]);
/* Computing 2nd power */
    d__1 = yt[10] - cn * xt[10];
/* Computing 2nd power */
    d__2 = xt[10] - an;
/* Computing 2nd power */
    d__3 = an;
    bn = sqrt(d__1 * d__1 / (1. - d__2 * d__2 / (d__3 * d__3)));
    for (j = 1; j <= 10; ++j) {
/* Computing 2nd power */
      d__1 = xt[j - 1] - an;
/* Computing 2nd power */
      d__2 = an;
      yt[j - 1] = bn * sqrt(1. - d__1 * d__1 / (d__2 * d__2)) + cn * xt[j - 1];
      if (xt[j - 1] <= e) {
	goto L510;
      }
/* Computing 2nd power */
      d__1 = bn;
/* Computing 2nd power */
      d__2 = an;
      ytp[j - 1] =
	d__1 * d__1 * (an - xt[j - 1]) / (d__2 * d__2) / (yt[j - 1] - cn * xt[j - 1]) + cn;
/* Computing 4th power */
      d__1 = bn, d__1 *= d__1;
/* Computing 2nd power */
      d__2 = an;
/* Computing 3rd power */
      d__3 = yt[j - 1] - cn * xt[j - 1];
      ytpp[j - 1] = -(d__1 * d__1) / (d__2 * d__2) / (d__3 * (d__3 * d__3));
    L510:
      ;
    }
/* Computing 2nd power */
    d__1 = bn;
    rnp = d__1 * d__1 / an;
    spline_(c__201, xt, yt, bspline, cspline, dspline);
/* update it */
  }

/* ..... Print column headings for the columns of data ................... */
  x = 0.;
  ali = fabs(cli[1]);
  x = 2.5e-4;
  xl[0] = 0.;
L550:
/* jump to here from above when I > 1 */
  yupr[i__ - 1] = yup;
  ylpr[i__ - 1] = ylp;
/*     Interpolate for thickness and derivatives at desired values of X: */
  spline_(c__201, xt, yt, bspline, cspline, dspline);
  seval3_(c__201, &x, xt, yt, bspline, cspline, dspline, &y, &yp, &ypp, &dummy);
/*     Compute camberline: */
  a[1] = as;
  cli[1] = clis;
  l = 0;
L600:
  ++l;
  a[1] = a[l];
  cli[1] = cli[l];
  xc = x * chd;
  yc = y * chd;
  xll = x * log(x);
  q = 1.;
  if ((d__1 = 1. - a[1], fabs(d__1)) < e && (d__2 = 1. - x, fabs(d__2)) < e) {
    g = 0.;
    h__ = 0.;
    q = 0.;
    z__ = 0.;
    z1 = -1e11;
    z2 = -1e11;
  } else if (a[1] < e && 1. - x < e) {
    g = -.25;
    h__ = -.5;
    q = 1.;
    z__ = -.25;
    z1 = 0.;
    z2 = -1e11;
  } else if ((d__1 = a[1] - x, fabs(d__1)) < e) {
/* Computing 2nd power */
    d__1 = 1. - x;
/* Computing 2nd power */
    d__2 = 1. - x;
    z__ = d__1 * d__1 * -.5 * log(1. - x) + d__2 * d__2 * .25;
    z1 = (1. - x) * -.5 * (-log(1. - x) - 1.) + (1. - x) * .5 * log(1. - x) - (1. - x) * .5;
    z2 = -log(1. - x) - .5;
/* Computing 2nd power */
    d__1 = a[1];
    g = -(d__1 * d__1 * (log(a[1]) * .5 - .25) + .25) / (1. - a[1]);
/* Computing 2nd power */
    d__1 = 1. - a[1];
/* Computing 2nd power */
    d__2 = 1. - a[1];
    h__ = (d__1 * d__1 * .5 * log(1. - a[1]) - d__2 * d__2 * .25) / (1. - a[1]) + g;
  } else if ((d__1 = 1. - x, fabs(d__1)) < e) {
/* Computing 2nd power */
    d__1 = a[1];
    g = -(d__1 * d__1 * (log(a[1]) * .5 - .25) + .25) / (1. - a[1]);
/* Computing 2nd power */
    d__1 = 1. - a[1];
/* Computing 2nd power */
    d__2 = 1. - a[1];
    h__ = (d__1 * d__1 * .5 * log(1. - a[1]) - d__2 * d__2 * .25) / (1. - a[1]) + g;
/* Computing 2nd power */
    d__2 = a[1] - 1.;
/* Computing 2nd power */
    d__3 = a[1] - 1.;
    z__ = d__2 * d__2 * .5 * log((d__1 = a[1] - 1., fabs(d__1))) - d__3 * d__3 * .25;
    z1 = -(a[1] - 1.) * log((d__1 = a[1] - 1., fabs(d__1)));
    z2 = -1e11;
  } else if ((d__1 = a[1] - 1., fabs(d__1)) < e) {
    g = 0.;
    h__ = 0.;
    q = 0.;
    z__ = -(1. - x) * log(1. - x);
    z1 = log(1. - x) + 1.;
    z2 = -1. / (1. - x);
  } else {
    v = -(a[1] - x) / (d__1 = a[1] - x, fabs(d__1));
    omxl = (1. - x) * log(1. - x);
    amxl = (a[1] - x) * log((d__1 = a[1] - x, fabs(d__1)));
    omxl1 = -log(1. - x) - 1.;
    amxl1 = -log((d__1 = a[1] - x, fabs(d__1))) - 1.;
    omxl2 = 1. / (1. - x);
    amxl2 = 1. / (a[1] - x);
/* Computing 2nd power */
    d__1 = a[1] - x;
/* Computing 2nd power */
    d__2 = 1. - x;
    z__ = (a[1] - x) * .5 * amxl - (1. - x) * .5 * omxl - d__1 * d__1 * .25 + d__2 * d__2 * .25;
    z1 = ((a[1] - x) * amxl1 - amxl - (1. - x) * omxl1 + omxl + (a[1] - x)
	  - (1. - x)) * .5;
    z2 = (a[1] - x) * .5 * amxl2 - amxl1 - (1. - x) * .5 * omxl2 + omxl1;
    if (a[1] <= e) {
      g = -.25;
      h__ = -.5;
    } else {
      g = -(a[1] * a[1] * (log(a[1]) * .5 - .25) + .25) / (1. - a[1]);
/* Computing 2nd power */
      d__1 = 1. - a[1];
/* Computing 2nd power */
      d__2 = 1. - a[1];
      h__ = (d__1 * d__1 * .5 * log(1. - a[1]) - d__2 * d__2 * .25) / (1. - a[1]) + g;
    }
  }
  ycmb[l - 1] = cli[1] * (z__ / (1. - q * a[1]) - xll + g - h__ * x) /
    3.14159265 / (a[1] + 1.) / 2.;
  xsv = x;
  if (x < .005) {
    x = .005;
  }
  tanth[l - 1] = cli[1] * (z1 / (1. - q * a[1]) - 1. - log(x) - h__) /
    3.14159265 / (a[1] + 1.) / 2.;
  x = xsv;
  if (if6xa == 1) {
    tanth[l - 1] = -2.;
  }
  if (x <= .005) {
    ycp2[l - 1] = 0.;
  } else if ((d__1 = 1. - x, fabs(d__1)) <= e) {
    ycp2[l - 1] = 1. / e;
  } else {
    pia = (a[1] + 1.) * 3.14159265 * 2;
    ycp2[l - 1] = cli[1] * (z2 / (1. - q * a[1]) - 1. / x) / pia;
  }
/*        Modified camberline option: */
  if (icamber > 100) {
    ycmb[l - 1] *= .97948;
    tanth[l - 1] *= .97948;
    if (tanth[l - 1] <= cli[1] * -.24521) {
      ycmb[l - 1] = cli[1] * .24521 * (1. - x);
    }
    if (tanth[l - 1] <= cli[1] * -.24521) {
      ycp2[l - 1] = 0.;
    }
    if (tanth[l - 1] <= cli[1] * -.24521) {
      tanth[l - 1] = cli[1] * -.24521;
    }
    if (tanth[l - 1] <= cli[1] * -.24521) {
      if6xa = 1;
    }
  }
  if (icky > 1 && l < icky) {
    goto L600;
  }
  if (icky == 1) {
    goto L620;
  }
  i__1 = icky;
  for (j = 2; j <= i__1; ++j) {
    ycmb[0] += ycmb[j - 1];
    tanth[0] += tanth[j - 1];
    ycp2[0] += ycp2[j - 1];
/* L610: */
  }
L620:
/* Computing 2nd power */
  d__1 = tanth[0];
  f = sqrt(d__1 * d__1 + 1.);
/* Computing 2nd power */
  d__1 = f;
  thp = ycp2[0] / (d__1 * d__1);
  sinth = tanth[0] / f;
  costh = 1. / f;
/*     Camberline and derivatives computed: */
  ++i__;
/*     Combine thickness distributuion and camberline: */
  xu[i__ - 1] = x - y * sinth;
  yu[i__ - 1] = ycmb[0] + y * costh;
  xl[i__ - 1] = x + y * sinth;
  yl[i__ - 1] = ycmb[0] - y * costh;
  if (x >= .815) {
    if (iprofile > 100) {
      if (x <= .825) {
	x2 = 1.;
	x1 = xu[i__ - 1];
	y2 = 0.;
	y1 = yu[i__ - 1];
	s1 = (y2 - y1) / (x2 - x1);
	s2 = (y2 - yl[i__ - 1]) / (x2 - xl[i__ - 1]);
	b1 = y2 - s1 * x2;
	b2 = y2 - s2 * x2;
      }
      yu[i__ - 1] = s1 * xu[i__ - 1] + b1;
      yl[i__ - 1] = s2 * xl[i__ - 1] + b2;
    }
  }
/*       Multiply by chord: */
  xuc = xu[i__ - 1] * chd;
  yuc = yu[i__ - 1] * chd;
  xlc = xl[i__ - 1] * chd;
  ylc = yl[i__ - 1] * chd;
  if (ali > e || icky != 1) {
/*        Find local slope of cambered profile: */
    yup = (tanth[0] * f + yp - tanth[0] * y * thp) / (f - yp * tanth[0] - y * thp);
    ylp = (tanth[0] * f - yp + tanth[0] * y * thp) / (f + yp * tanth[0] + y * thp);
  }
/*     Find X increment: */
  if (x <= .0975) {
    frac = .25;
  }
  if (x <= .01225) {
    frac = .025;
  }

/* ..... Store scaled profile in appropriate arrays ...................... */
  axuc[i__ - 1] = xuc;
  ayuc[i__ - 1] = yuc;
  if (ali >= 1e-10 || icky != 1) {
    ax[i__ - 1] = x;
    axlc[i__ - 1] = xlc;
    aylc[i__ - 1] = ylc;
  }
/*     Increment X and return to start of X loop: */
  x += frac * dx;
  frac = 1.;
  xau[i__ - 1] = xuc;
  yau[i__ - 1] = yuc;
  xal[i__ - 1] = xlc;
  yal[i__ - 1] = ylc;
  if (x <= 1.) {
    goto L300;
  }
/*     Calculate derivatives of final coordinates; tabulate and save results. */
/* keep going back to 300 until x>1 */
  if (ali >= e || icky != 1) {
    spline_(i__, xu, yu, bspline, cspline, dspline);
    i__1 = i__;
    for (j = 1; j <= i__1; ++j) {
      seval3_(i__, &xu[j - 1], xu, yu, bspline, cspline, dspline, &dummy, &ayup[j - 1], &dummy,
	      &dummy);
    }
    spline_(i__, xl, yl, bspline, cspline, dspline);
    i__1 = i__;
    for (j = 1; j <= i__1; ++j) {
      seval3_(i__, &xl[j - 1], xl, yl, bspline, cspline, dspline, &dummy, &aylp[j - 1], &dummy,
	      &dummy);
    }
    *nout = i__;
/*        Store results in output array XYOUT, starting at upper side TE, */
/*        then moving around the LE and back along the lower side */
    i__1 = i__;
    for (j = 1; j <= i__1; ++j) {
      xyout[j] = xu[i__ + 1 - j - 1];
      xyout[i__ + j] = xl[j - 1];
      xyout[(i__ << 1) + j] = yu[i__ + 1 - j - 1];
      xyout[i__ * 3 + j] = yl[j - 1];
    }
  } else {
    spline_(i__, xu, yu, bspline, cspline, dspline);
    i__1 = i__;
    for (j = 1; j <= i__1; ++j) {
      seval3_(i__, &xu[j - 1], xu, yu, bspline, cspline, dspline, &dummy, &ayp[j - 1], &aypp[j - 1],
	      &dummy);
    }
    *nout = i__;
/*        Store results in output array XYOUT */
    i__1 = i__;
    for (j = 1; j <= i__1; ++j) {
      xyout[j] = xu[i__ + 1 - j - 1];
      xyout[i__ + j] = xu[j - 1];
      xyout[(i__ << 1) + j] = yu[i__ + 1 - j - 1];
      xyout[i__ * 3 + j] = -yu[j - 1];
    }
  }
  
  if (*nout > 5) {
    double y0 = xyout[5];
    if (y0 == y0) 
      return NACA6_SUCCESS;
    else
      return NACA6_NOTCONVERGED;
  } else {
    return NACA6_NOTCONVERGED;
  }
}				/* naca6_ */

/* Subroutine */ int
phep63_(double *phi, double *eps)
{
  /* Initialized data */

  static double phild[201] = { 0., .01569, .03139, .04708, .06278, .07848,
    .09419, .1099, .12562, .14135, .15708, .17277, .18846, .20416, .21987,
    .23558, .25129, .26701, .28273, .29844, .31416, .32987, .34559, .3613,
    .377, .39271, .40842, .42412, .43983, .45553, .47124, .48695, .50266,
    .51837, .53407, .54978, .56549, .5812, .59691, .61261, .62832, .64403,
    .65974, .67545, .69116, .70687, .72257, .73828, .75399, .76969, .7854,
    .80111, .81682, .83253, .84824, .86395, .87965, .89536, .91107, .92677,
    .94248, .95819, .9739, .98961, 1.00532, 1.02103, 1.03673, 1.05244,
    1.06815, 1.08385, 1.09956, 1.11527, 1.13098, 1.14669, 1.1624, 1.17811,
    1.19381, 1.20952, 1.22523, 1.24093, 1.25664, 1.27235, 1.28806, 1.30376,
    1.31947, 1.33518, 1.35089, 1.36659, 1.3823, 1.39801, 1.41372, 1.42942,
    1.44513, 1.46084, 1.47654, 1.49225, 1.50796, 1.52367, 1.53938, 1.55509,
    1.5708, 1.5865, 1.60221, 1.61791, 1.63362, 1.64933, 1.66504, 1.68075,
    1.69646, 1.71217, 1.72788, 1.74358, 1.75929, 1.775, 1.7907, 1.80641,
    1.82212, 1.83783, 1.85354, 1.86925, 1.88496, 1.90067, 1.91637, 1.93208,
    1.94779, 1.9635, 1.97921, 1.99491, 2.01062, 2.02633, 2.04204, 2.05775,
    2.07346, 2.08917, 2.10487, 2.12058, 2.13629, 2.152, 2.1677, 2.18341,
    2.19911, 2.21482, 2.23054, 2.24625, 2.26196, 2.27767, 2.29338, 2.30908,
    2.32479, 2.34049, 2.35619, 2.37191, 2.38762, 2.40334, 2.41905, 2.43476,
    2.45046, 2.46617, 2.48187, 2.49757, 2.51327, 2.52899, 2.5447, 2.56042,
    2.57613, 2.59184, 2.60754, 2.62325, 2.63895, 2.65465, 2.67035, 2.68607,
    2.70178, 2.71749, 2.7332, 2.74891, 2.76462, 2.78032, 2.79603, 2.81173,
    2.82743, 2.84314, 2.85885, 2.87456, 2.89027, 2.90598, 2.92169, 2.9374,
    2.9531, 2.96881, 2.98451, 3.00022, 3.01593, 3.03164, 3.04735, 3.06305,
    3.07876, 3.09447, 3.11018, 3.12588, 3.14159
  };
  static double epsld[201] = { 0., .00164, .00327, .00487, .00641, .00789,
    .00928, .01057, .01174, .01278, .01367, .01439, .01497, .01542, .01576,
    .01601, .01619, .01632, .01642, .01651, .01661, .01673, .01688, .01705,
    .01725, .01747, .01771, .01797, .01824, .01853, .01884, .01916, .01949,
    .01984, .0202, .02058, .02097, .02137, .02179, .02223, .02268, .02315,
    .02363, .02413, .02464, .02517, .02571, .02626, .02683, .02741, .02801,
    .02862, .02924, .02988, .03052, .03118, .03185, .03253, .03323, .03393,
    .03465, .03538, .03611, .03686, .03762, .03839, .03917, .03995, .04075,
    .04156, .04237, .04319, .04402, .04486, .04571, .04657, .04743, .04831,
    .04919, .05008, .05098, .05189, .0528, .05372, .05464, .05556, .05648,
    .0574, .05831, .05921, .06011, .06099, .06187, .06273, .06357, .0644,
    .06522, .06602, .06681, .06757, .06832, .06905, .06976, .07044, .07111,
    .07176, .07238, .07298, .07356, .07411, .07464, .07514, .07562, .07607,
    .0765, .0769, .07727, .07761, .07793, .07822, .07848, .07871, .07891,
    .07908, .07922, .07933, .07941, .07945, .07946, .07944, .07938, .07929,
    .07916, .079, .0788, .07856, .07829, .07799, .07764, .07726, .07685, .0764,
    .07591, .07539, .07483, .07423, .07359, .07293, .07222, .07148, .0707,
    .06989, .06904, .06815, .06723, .06628, .06529, .06427, .06322, .06214,
    .06103, .05989, .05871, .05751, .05628, .05502, .05374, .05243, .05109,
    .04973, .04834, .04693, .04549, .04404, .04256, .04106, .03955, .03802,
    .03647, .03491, .03333, .03174, .03014, .02853, .0269, .02527, .02363,
    .02198, .02032, .01865, .01698, .0153, .01361, .01192, .01023, .00853,
    .00683, .00512, .00342, .00171, 0.
  };

  int i__;

  /* Parameter adjustments */
  --eps;
  --phi;

  /* Function Body */
  for (i__ = 1; i__ <= 201; ++i__) {
/* OK */
    phi[i__] = phild[i__ - 1];
    eps[i__] = epsld[i__ - 1];
  }
  return 0;
}				/* phep63_ */

/* ----------------------------------- End of subroutine PHEP63 */
/* Subroutine */ int
phps63_(double *phi, double *psi)
{
  /* Initialized data */

  static double phild[201] = { 0., .01571, .03142, .04713, .06284, .07855,
    .09426, .10997, .12567, .14138, .15708, .17279, .1885, .2042, .21991,
    .23561, .25132, .26703, .28274, .29845, .31416, .32986, .34556, .36127,
    .37698, .39268, .40839, .42411, .43982, .45553, .47124, .48695, .50266,
    .51837, .53408, .54978, .56549, .5812, .59691, .61261, .62832, .64403,
    .65974, .67544, .69115, .70686, .72257, .73828, .75398, .76969, .7854,
    .80111, .81682, .83252, .84823, .86394, .87965, .89536, .91106, .92677,
    .94248, .95819, .9739, .98961, 1.00531, 1.02102, 1.03673, 1.05244,
    1.06815, 1.08385, 1.09956, 1.11527, 1.13098, 1.14669, 1.1624, 1.17811,
    1.19381, 1.20952, 1.22523, 1.24093, 1.25664, 1.27235, 1.28807, 1.30378,
    1.31949, 1.3352, 1.3509, 1.36661, 1.38231, 1.39802, 1.41372, 1.42944,
    1.44515, 1.46086, 1.47657, 1.49228, 1.50799, 1.52369, 1.5394, 1.5551,
    1.5708, 1.58652, 1.60223, 1.61794, 1.63365, 1.64936, 1.66507, 1.68077,
    1.69648, 1.71218, 1.72788, 1.74359, 1.75931, 1.77502, 1.79073, 1.80644,
    1.82214, 1.83785, 1.85355, 1.86926, 1.88496, 1.90067, 1.91638, 1.93209,
    1.9478, 1.96351, 1.97922, 1.99493, 2.01063, 2.02634, 2.04204, 2.05775,
    2.07346, 2.08916, 2.10487, 2.12058, 2.13628, 2.15199, 2.1677, 2.1834,
    2.19911, 2.21481, 2.23052, 2.24623, 2.26193, 2.27764, 2.29335, 2.30906,
    2.32477, 2.34048, 2.35619, 2.37189, 2.38759, 2.4033, 2.419, 2.43471,
    2.45042, 2.46613, 2.48184, 2.49755, 2.51327, 2.52897, 2.54467, 2.56037,
    2.57608, 2.59179, 2.60749, 2.62321, 2.63892, 2.65463, 2.67035, 2.68605,
    2.70175, 2.71745, 2.73316, 2.74887, 2.76458, 2.78029, 2.796, 2.81171,
    2.82743, 2.84313, 2.85884, 2.87454, 2.89025, 2.90596, 2.92167, 2.93737,
    2.95309, 2.9688, 2.98451, 3.00022, 3.01592, 3.03163, 3.04734, 3.06305,
    3.07875, 3.09446, 3.11017, 3.12588, 3.14159
  };
  static double psild[201] = { .15066, .15058, .15035, .14999, .1495, .14891,
    .14823, .14748, .14668, .14583, .14497, .1441, .14323, .14238, .14155,
    .14074, .13998, .13927, .13862, .13804, .13753, .13711, .13676, .13648,
    .13627, .1361, .13598, .1359, .13584, .13579, .13576, .13573, .1357,
    .13567, .13564, .13561, .13558, .13555, .13552, .1355, .13547, .13544,
    .13542, .13539, .13536, .13533, .13529, .13525, .13521, .13516, .13511,
    .13505, .13499, .13491, .13483, .13475, .13465, .13454, .13442, .13428,
    .13414, .13398, .13381, .13363, .13343, .13321, .13299, .13275, .13249,
    .13222, .13194, .13164, .13133, .131, .13065, .13028, .12988, .12947,
    .12903, .12857, .12808, .12756, .12702, .12644, .12584, .12521, .12455,
    .12385, .12313, .12238, .1216, .12079, .11994, .11907, .11817, .11724,
    .11628, .11529, .11428, .11324, .11218, .11109, .10998, .10884, .10768,
    .1065, .1053, .10407, .10283, .10157, .10029, .09899, .09767, .09634,
    .09499, .09363, .09224, .09085, .08944, .08801, .08657, .08512, .08365,
    .08217, .08068, .07917, .07766, .07614, .07461, .07307, .07153, .06998,
    .06842, .06687, .0653, .06374, .06217, .0606, .05904, .05747, .05591,
    .05435, .0528, .05125, .0497, .04817, .04664, .04512, .04362, .04213,
    .04065, .03919, .03774, .03631, .0349, .0335, .03213, .03077, .02943,
    .02811, .02682, .02555, .0243, .02308, .02188, .02071, .01956, .01844,
    .01735, .0163, .01527, .01428, .01331, .01239, .01149, .01062, .00979,
    .00899, .00823, .0075, .0068, .00614, .00551, .00491, .00435, .00382,
    .00332, .00286, .00244, .00205, .00169, .00137, .00108, 8.3e-4, 6.1e-4,
    4.2e-4, 2.7e-4, 1.5e-4, 7e-5, 2e-5, 0.
  };

  double b[201], c__[201], d__[201];
  int i__;
  double xx;

  /* Parameter adjustments */
  --psi;
  --phi;

  /* Function Body */
  spline_(c__201, phild, psild, b, c__, d__);
/* OK */
  for (i__ = 1; i__ <= 201; ++i__) {
    seval3_(c__201, &phi[i__], phild, psild, b, c__, d__, &psi[i__], &xx, &xx, &xx);
/* L202: */
  }
  return 0;
}				/* phps63_ */

/* ----------------------------------- End of subroutine PHPS63 */
/* Subroutine */ int
phep64_(double *phi, double *eps)
{
  /* Initialized data */

  static double phild[201] = { 0., .01568, .03136, .04705, .06274, .07843,
    .09414, .10985, .12558, .14132, .15708, .17274, .18842, .20411, .21981,
    .23552, .25124, .26697, .2827, .29843, .31416, .32987, .34558, .36129,
    .377, .39271, .40842, .42412, .43983, .45553, .47124, .48695, .50266,
    .51837, .53408, .54979, .56549, .5812, .59691, .61262, .62832, .64403,
    .65975, .67546, .69117, .70688, .72258, .73829, .75399, .7697, .7854,
    .80111, .81683, .83254, .84825, .86396, .87967, .89537, .91108, .92678,
    .94248, .9582, .97391, .98962, 1.00533, 1.02104, 1.03675, 1.05245,
    1.06816, 1.08386, 1.09956, 1.11528, 1.13099, 1.14671, 1.16242, 1.17813,
    1.19384, 1.20954, 1.22524, 1.24094, 1.25664, 1.27236, 1.28807, 1.30378,
    1.31949, 1.3352, 1.35091, 1.36661, 1.38232, 1.39802, 1.41372, 1.42942,
    1.44512, 1.46083, 1.47653, 1.49223, 1.50794, 1.52365, 1.53936, 1.55508,
    1.5708, 1.58649, 1.60218, 1.61788, 1.63358, 1.64929, 1.665, 1.68072,
    1.69643, 1.71216, 1.72788, 1.74358, 1.75928, 1.77498, 1.79068, 1.80639,
    1.8221, 1.83781, 1.85353, 1.86924, 1.88496, 1.90066, 1.91637, 1.93207,
    1.94778, 1.96349, 1.9792, 1.99491, 2.01062, 2.02633, 2.04204, 2.05775,
    2.07346, 2.08917, 2.10488, 2.12059, 2.1363, 2.152, 2.16771, 2.18341,
    2.19911, 2.21483, 2.23055, 2.24627, 2.26198, 2.2777, 2.2934, 2.30911,
    2.32481, 2.3405, 2.35619, 2.37192, 2.38765, 2.40337, 2.41909, 2.4348,
    2.45051, 2.46621, 2.4819, 2.49759, 2.51327, 2.52901, 2.54474, 2.56047,
    2.57619, 2.5919, 2.6076, 2.6233, 2.63899, 2.65467, 2.67035, 2.68609,
    2.70182, 2.71755, 2.73327, 2.74898, 2.76468, 2.78038, 2.79607, 2.81175,
    2.82743, 2.84316, 2.85889, 2.87461, 2.89032, 2.90603, 2.92173, 2.93743,
    2.95313, 2.96882, 2.98451, 3.00023, 3.01594, 3.03165, 3.04736, 3.06307,
    3.07878, 3.09448, 3.11018, 3.12589, 3.14159
  };
  static double epsld[201] = { 0., .00233, .00464, .00692, .00914, .01129,
    .01336, .01531, .01714, .01883, .02035, .02169, .02287, .0239, .0248,
    .02557, .02624, .02682, .02731, .02774, .02812, .02846, .02877, .02905,
    .02931, .02957, .02982, .03007, .03033, .0306, .0309, .03122, .03158,
    .03196, .03236, .0328, .03326, .03375, .03427, .03481, .03538, .03598,
    .0366, .03725, .03792, .03862, .03935, .0401, .04087, .04167, .0425,
    .04335, .04423, .04512, .04605, .04699, .04796, .04896, .04998, .05102,
    .05208, .05317, .05428, .05541, .05657, .05774, .05894, .06016, .0614,
    .06267, .06395, .06526, .06658, .06793, .06931, .0707, .07213, .07357,
    .07505, .07655, .07808, .07964, .08123, .08284, .08447, .08613, .0878,
    .08949, .09119, .0929, .09462, .09635, .09808, .0998, .10151, .10321,
    .10488, .10653, .10815, .10972, .11125, .11273, .11415, .11553, .11686,
    .11814, .11938, .12057, .12171, .12281, .12386, .12487, .12583, .12675,
    .12762, .12844, .12922, .12994, .13062, .13125, .13182, .13234, .13281,
    .13322, .13358, .13389, .13414, .13434, .13448, .13456, .13459, .13456,
    .13447, .13433, .13413, .13387, .13354, .13316, .13272, .13222, .13166,
    .13104, .13035, .1296, .12879, .12792, .12698, .12598, .12492, .1238,
    .12261, .12136, .12004, .11866, .11722, .11572, .11416, .11254, .11087,
    .10914, .10735, .1055, .1036, .10164, .09963, .09757, .09545, .09328,
    .09105, .08878, .08645, .08406, .08162, .07913, .0766, .07402, .07139,
    .06873, .06603, .06329, .06052, .0577, .05486, .05198, .04907, .04614,
    .04318, .0402, .0372, .03417, .03113, .02807, .02499, .02189, .01879,
    .01567, .01255, .00942, .00628, .00314, 0.
  };

  int j;

  /* Parameter adjustments */
  --eps;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  for (j = 1; j <= 201; ++j) {
/* OK */
    phi[j] = phild[j - 1];
    eps[j] = epsld[j - 1];
/* L201: */
  }
  return 0;
}				/* phep64_ */

/* + */
/* ----------------------------------- End of subroutine PHEP64 */
/* Subroutine */ int
phps64_(double *phi, double *psi)
{
  /* Initialized data */

  static double phild[201] = { 0., .01573, .03145, .04717, .06289, .07861,
    .09432, .11002, .12572, .1414, .15708, .1728, .18851, .20422, .21992,
    .23562, .25132, .26702, .28272, .29844, .31416, .32984, .34553, .36122,
    .37693, .39264, .40836, .42408, .4398, .45552, .47124, .48695, .50266,
    .51837, .53408, .54979, .56549, .5812, .59691, .61261, .62832, .64403,
    .65973, .67544, .69115, .70686, .72257, .73827, .75398, .76969, .7854,
    .80111, .81682, .83252, .84823, .86394, .87965, .89536, .91106, .92677,
    .94248, .95819, .9739, .98961, 1.00531, 1.02102, 1.03673, 1.05244,
    1.06815, 1.08385, 1.09956, 1.11527, 1.13098, 1.14669, 1.1624, 1.17811,
    1.19381, 1.20952, 1.22523, 1.24093, 1.25664, 1.27236, 1.28808, 1.30379,
    1.31951, 1.33522, 1.35093, 1.36663, 1.38233, 1.39803, 1.41372, 1.42946,
    1.44519, 1.46091, 1.47663, 1.49234, 1.50804, 1.52374, 1.53943, 1.55512,
    1.5708, 1.58653, 1.60226, 1.61798, 1.6337, 1.64941, 1.66511, 1.68081,
    1.69651, 1.7122, 1.72788, 1.74361, 1.75933, 1.77505, 1.79076, 1.80647,
    1.82218, 1.83788, 1.85357, 1.86927, 1.88496, 1.90068, 1.9164, 1.93211,
    1.94782, 1.96353, 1.97924, 1.99494, 2.01064, 2.02634, 2.04204, 2.05775,
    2.07346, 2.08917, 2.10488, 2.12059, 2.13629, 2.152, 2.1677, 2.18341,
    2.19911, 2.21481, 2.23051, 2.24622, 2.26192, 2.27763, 2.29334, 2.30905,
    2.32476, 2.34047, 2.35619, 2.37188, 2.38757, 2.40327, 2.41897, 2.43468,
    2.45039, 2.4661, 2.48182, 2.49754, 2.51327, 2.52895, 2.54464, 2.56034,
    2.57604, 2.59174, 2.60745, 2.62317, 2.63889, 2.65462, 2.67035, 2.68603,
    2.70172, 2.71741, 2.73311, 2.74882, 2.76453, 2.78024, 2.79597, 2.8117,
    2.82743, 2.84312, 2.85881, 2.87451, 2.89021, 2.90591, 2.92163, 2.93734,
    2.95306, 2.96878, 2.98451, 3.00021, 3.01591, 3.03162, 3.04732, 3.06303,
    3.07874, 3.09445, 3.11016, 3.12588, 3.14159
  };
  static double psild[201] = { .25269, .25265, .25251, .25227, .25193,
    .25147, .2509, .2502, .24937, .24841, .2473, .24605, .24467, .24321, .2417,
    .24016, .23864, .23715, .23573, .23442, .23325, .23224, .23138, .23066,
    .23006, .22956, .22916, .22884, .22858, .22836, .22818, .22802, .22788,
    .22775, .22764, .22755, .22747, .2274, .22736, .22732, .2273, .22729,
    .2273, .22731, .22733, .22736, .22739, .22742, .22745, .22748, .22751,
    .22753, .22755, .22756, .22756, .22755, .22753, .22751, .22747, .22742,
    .22736, .22729, .2272, .22709, .22697, .22683, .22668, .2265, .2263,
    .22608, .22584, .22557, .22528, .22497, .22462, .22426, .22386, .22345,
    .223, .22253, .22203, .2215, .22094, .22034, .21969, .21899, .21823,
    .21741, .21652, .21554, .21449, .21334, .21211, .2108, .2094, .20794,
    .20641, .20481, .20315, .20143, .19966, .19784, .19596, .19405, .19209,
    .19008, .18804, .18596, .18384, .18169, .1795, .17727, .17501, .17273,
    .17041, .16806, .16569, .16329, .16087, .15843, .15596, .15347, .15095,
    .14842, .14587, .1433, .14072, .13812, .13551, .13288, .13024, .12759,
    .12492, .12225, .11957, .11688, .11418, .11149, .10878, .10608, .10338,
    .10068, .09798, .09529, .0926, .08992, .08725, .08459, .08195, .07932,
    .07671, .07412, .07156, .06901, .06649, .06399, .06152, .05908, .05667,
    .05428, .05193, .04962, .04734, .0451, .04289, .04073, .0386, .03651,
    .03447, .03247, .03051, .02861, .02675, .02494, .02318, .02148, .01983,
    .01825, .01672, .01525, .01385, .01252, .01125, .01006, .00892, .00786,
    .00686, .00593, .00506, .00426, .00353, .00287, .00227, .00174, .00128,
    8.9e-4, 5.7e-4, 3.2e-4, 1.4e-4, 4e-5, 0.
  };

  double b[201], c__[201], d__[201];
  int j;
  double xx;

/*   -------------------------------------------------------------------- */
/*     PURPOSE - */


/* *********************************************************************** */
/*     A R G U M E N T S                                                * */
/* *********************************************************************** */
/* *********************************************************************** */
/*     L O C A L   V A R I A B L E S                                    * */
/* *********************************************************************** */
/* *********************************************************************** */
/*     L O C A L   A R R A Y S                                          * */
/* *********************************************************************** */
  /* Parameter adjustments */
  --psi;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  spline_(c__201, phild, psild, b, c__, d__);

/* OK */
  for (j = 1; j <= 201; ++j) {
    seval3_(c__201, &phi[j], phild, psild, b, c__, d__, &psi[j], &xx, &xx, &xx);
/* !!      CALL LCSFIT (201, PHILD, PSILD, .TRUE., 'B', 201, PHI, PSI, PSI) */
/* L202: */
  }
  return 0;
}				/* phps64_ */

/* + */
/* ----------------------------------- End of subroutine PHPS64 */
/* Subroutine */ int
phep65_(double *phi, double *eps)
{
  /* Initialized data */

  static double phild[251] = { 0., .01484, .02967, .04451, .05935, .0742,
    .08904, .10389, .11875, .13361, .14848, .15175, .15503, .1583, .16157,
    .16485, .16812, .1714, .17468, .17795, .18123, .18623, .19123, .19623,
    .20123, .20623, .21124, .21624, .22124, .22625, .23125, .24072, .25018,
    .25964, .2691, .27856, .28802, .29748, .30694, .3164, .32586, .33918,
    .3525, .36582, .37915, .39247, .4058, .41912, .43245, .44577, .4591,
    .46962, .48015, .49068, .5012, .51173, .52226, .53278, .54331, .55384,
    .56436, .57344, .58252, .5916, .60068, .60976, .61884, .62792, .637,
    .64607, .65515, .67086, .68656, .70226, .71797, .73367, .74937, .76508,
    .78078, .79648, .81218, .8259, .83962, .85333, .86705, .88077, .89448,
    .9082, .92192, .93563, .94934, .96192, .97449, .98706, .99964, 1.01221,
    1.02478, 1.03735, 1.04992, 1.06249, 1.07506, 1.08689, 1.09872, 1.11055,
    1.12238, 1.13421, 1.14603, 1.15786, 1.16969, 1.18151, 1.19334, 1.20469,
    1.21605, 1.2274, 1.23876, 1.25011, 1.26146, 1.27281, 1.28416, 1.29551,
    1.30686, 1.31793, 1.329, 1.34007, 1.35114, 1.36221, 1.37328, 1.38434,
    1.39541, 1.40648, 1.41754, 1.42845, 1.43935, 1.45025, 1.46116, 1.47206,
    1.48296, 1.49387, 1.50477, 1.51567, 1.52657, 1.53737, 1.54816, 1.55896,
    1.56975, 1.58055, 1.59134, 1.60214, 1.61293, 1.62373, 1.63452, 1.64525,
    1.65598, 1.66671, 1.67744, 1.68817, 1.6989, 1.70963, 1.72036, 1.73109,
    1.74182, 1.75254, 1.76325, 1.77396, 1.78468, 1.79539, 1.80611, 1.81683,
    1.82754, 1.83826, 1.84898, 1.85972, 1.87046, 1.8812, 1.89194, 1.90268,
    1.91343, 1.92417, 1.93491, 1.94566, 1.9564, 1.96732, 1.97823, 1.98915,
    2.00006, 2.01098, 2.0219, 2.03282, 2.04373, 2.05465, 2.06557, 2.07682,
    2.08806, 2.09931, 2.11056, 2.1218, 2.13305, 2.14429, 2.15554, 2.16679,
    2.17803, 2.18979, 2.20155, 2.2133, 2.22506, 2.23681, 2.24856, 2.26032,
    2.27207, 2.28382, 2.29557, 2.30822, 2.32088, 2.33353, 2.34618, 2.35883,
    2.37148, 2.38412, 2.39677, 2.40942, 2.42206, 2.43618, 2.45029, 2.4644,
    2.47851, 2.4926, 2.50669, 2.52077, 2.53483, 2.54888, 2.56292, 2.57978,
    2.59661, 2.61343, 2.63024, 2.64704, 2.66383, 2.68062, 2.6974, 2.7142,
    2.731, 2.77185, 2.81276, 2.85373, 2.89475, 2.93582, 2.97693, 3.01807,
    3.05923, 3.10041, 3.14159
  };
  static double epsld[251] = { 0., .00156, .00312, .00466, .00618, .00767,
    .00913, .01054, .01191, .01322, .01446, .01473, .01499, .01524, .01549,
    .01572, .01595, .01616, .01636, .01655, .01671, .01693, .01712, .01727,
    .0174, .0175, .01759, .01766, .01773, .0178, .01788, .01804, .01822,
    .01842, .01863, .01884, .01903, .0192, .01934, .01945, .0195, .01949,
    .01938, .01919, .01895, .01866, .01836, .01805, .01775, .01749, .01728,
    .01716, .01708, .01703, .01702, .01704, .01708, .01714, .01722, .01731,
    .01741, .01751, .0176, .01771, .01782, .01794, .01806, .0182, .01834,
    .0185, .01867, .01898, .01933, .01971, .02012, .02055, .021, .02147,
    .02196, .02246, .02298, .02343, .0239, .02437, .02485, .02534, .02584,
    .02636, .02689, .02744, .028, .02853, .02907, .02963, .0302, .03078,
    .03137, .03198, .03259, .03321, .03385, .03445, .03506, .03568, .03631,
    .03694, .03759, .03824, .0389, .03957, .04025, .04091, .04158, .04225,
    .04294, .04363, .04434, .04505, .04577, .0465, .04724, .04798, .04872,
    .04947, .05023, .05099, .05177, .05255, .05334, .05414, .05495, .05575,
    .05655, .05737, .05819, .05901, .05984, .06067, .0615, .06233, .06316,
    .06399, .06481, .06563, .06645, .06727, .06808, .06889, .06969, .07049,
    .07127, .07205, .07282, .07358, .07433, .07507, .0758, .07651, .07722,
    .07792, .07861, .07928, .07994, .08058, .08121, .08182, .08242, .08299,
    .08355, .08408, .08459, .08508, .08555, .08599, .08641, .0868, .08718,
    .08754, .08787, .08819, .08848, .08876, .08903, .08927, .08949, .08969,
    .08988, .09004, .09019, .09031, .09041, .0905, .09056, .09061, .09062,
    .09061, .09058, .09052, .09043, .09032, .09017, .08999, .08977, .08953,
    .08925, .08894, .08861, .08824, .08785, .08743, .08698, .08647, .08593,
    .08536, .08477, .08414, .08348, .08279, .08208, .08134, .08056, .07967,
    .07872, .07772, .07666, .07552, .07429, .07297, .07155, .07, .06833,
    .06615, .0638, .06132, .05873, .05606, .05334, .0506, .04788, .04519,
    .04258, .03664, .03122, .02628, .02176, .01759, .01371, .01008, .00662,
    .00328, 0.
  };

  double b[251], c__[251], d__[251];
  int j;
  double xx;
  /* Parameter adjustments */
  --eps;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  spline_(c__251, phild, epsld, b, c__, d__);

/* OK */
  for (j = 1; j <= 201; ++j) {
    phi[j] = (double) (j - 1) * 3.141592654 / 200.;
    seval3_(c__251, &phi[j], phild, epsld, b, c__, d__, &eps[j], &xx, &xx, &xx);
/* !!  201 CALL LCSFIT (251, PHILD, EPSLD, .TRUE., 'B', 201, PHI, EPS, EPS) */
/* L201: */
  }
  return 0;
}				/* phep65_ */

/* + */
/* ----------------------------------- End of subroutine PHEP65 */
/* Subroutine */ int
phps65_(double *phi, double *psi)
{
  /* Initialized data */

  static double phild[251] = { 0., .01486, .02972, .04458, .05944, .07429,
    .08914, .10398, .11882, .13365, .14848, .15176, .15503, .15831, .16159,
    .16486, .16814, .17141, .17469, .17796, .18123, .18624, .19124, .19624,
    .20124, .20624, .21124, .21625, .22125, .22625, .23125, .2407, .25015,
    .2596, .26906, .27852, .28798, .29744, .30691, .31638, .32586, .33917,
    .35249, .36581, .37913, .39246, .40578, .41911, .43244, .44577, .4591,
    .46962, .48015, .49068, .50121, .51173, .52226, .53279, .54331, .55384,
    .56436, .57344, .58252, .5916, .60068, .60976, .61884, .62791, .63699,
    .64607, .65515, .67085, .68656, .70226, .71796, .73366, .74937, .76507,
    .78077, .79648, .81218, .8259, .83961, .85333, .86704, .88076, .89448,
    .90819, .92191, .93563, .94934, .96192, .97449, .98706, .99963, 1.0122,
    1.02477, 1.03734, 1.04992, 1.06249, 1.07506, 1.08689, 1.09872, 1.11054,
    1.12237, 1.1342, 1.14603, 1.15786, 1.16968, 1.18151, 1.19334, 1.20469,
    1.21604, 1.2274, 1.23875, 1.2501, 1.26145, 1.27281, 1.28416, 1.29551,
    1.30686, 1.31793, 1.329, 1.34007, 1.35114, 1.36221, 1.37327, 1.38434,
    1.39541, 1.40647, 1.41754, 1.42845, 1.43935, 1.45026, 1.46116, 1.47207,
    1.48297, 1.49387, 1.50477, 1.51567, 1.52657, 1.53737, 1.54817, 1.55897,
    1.56977, 1.58057, 1.59136, 1.60215, 1.61295, 1.62374, 1.63452, 1.64526,
    1.656, 1.66673, 1.67746, 1.6882, 1.69892, 1.70965, 1.72038, 1.7311,
    1.74182, 1.75255, 1.76327, 1.77399, 1.78471, 1.79542, 1.80614, 1.81685,
    1.82756, 1.83828, 1.84898, 1.85973, 1.87048, 1.88122, 1.89197, 1.90271,
    1.91345, 1.92419, 1.93493, 1.94567, 1.9564, 1.96732, 1.97824, 1.98916,
    2.00008, 2.011, 2.02191, 2.03283, 2.04374, 2.05466, 2.06557, 2.07682,
    2.08807, 2.09932, 2.11056, 2.12181, 2.13305, 2.1443, 2.15554, 2.16679,
    2.17803, 2.18979, 2.20154, 2.2133, 2.22505, 2.2368, 2.24856, 2.26031,
    2.27206, 2.28382, 2.29557, 2.30822, 2.32086, 2.33351, 2.34616, 2.35881,
    2.37146, 2.38411, 2.39676, 2.40941, 2.42206, 2.43614, 2.45023, 2.46431,
    2.47839, 2.49248, 2.50656, 2.52065, 2.53474, 2.54883, 2.56292, 2.57971,
    2.59651, 2.6133, 2.6301, 2.64691, 2.66372, 2.68053, 2.69735, 2.71417,
    2.731, 2.77194, 2.81292, 2.85393, 2.89497, 2.93604, 2.97712, 3.01823,
    3.05934, 3.10046, 3.14159
  };
  static double psild[251] = { .17464, .17457, .17437, .17404, .1736, .17303,
    .17236, .17159, .17073, .16977, .16872, .16848, .16824, .16799, .16773,
    .16747, .16721, .16694, .16666, .16638, .16609, .16563, .16516, .16468,
    .1642, .16371, .16321, .16272, .16223, .16175, .16127, .16041, .15959,
    .15881, .15808, .1574, .15676, .15616, .15562, .15512, .15468, .15413,
    .15368, .15331, .15301, .15278, .15261, .15249, .15241, .15237, .15235,
    .15235, .15236, .15238, .15241, .15245, .15251, .15257, .15264, .15272,
    .15281, .15289, .15298, .15308, .15318, .15328, .15339, .1535, .15361,
    .15372, .15383, .15402, .15422, .1544, .15459, .15477, .15496, .15513,
    .15531, .15548, .15564, .15578, .15592, .15605, .15618, .1563, .15642,
    .15653, .15663, .15673, .15682, .1569, .15697, .15704, .15709, .15714,
    .15719, .15723, .15726, .15728, .1573, .15731, .15731, .1573, .15729,
    .15727, .15724, .15721, .15716, .15711, .15704, .15697, .15689, .1568,
    .15671, .1566, .15648, .15636, .15623, .15609, .15594, .15578, .15562,
    .15544, .15526, .15506, .15484, .15462, .15437, .15411, .15383, .15354,
    .15322, .15289, .15254, .15216, .15177, .15135, .15091, .15046, .14998,
    .14948, .14896, .14842, .14785, .14726, .14665, .14602, .14536, .14467,
    .14396, .14323, .14247, .14169, .14088, .14005, .1392, .13833, .13744,
    .13652, .13559, .13464, .13367, .13268, .13167, .13065, .12961, .12856,
    .12749, .12641, .12531, .1242, .12307, .12194, .12079, .11962, .11845,
    .11726, .11606, .11485, .11363, .11237, .11111, .10983, .10854, .10725,
    .10594, .10463, .10332, .10199, .10066, .09929, .09791, .09652, .09513,
    .09374, .09234, .09094, .08953, .08811, .08669, .0852, .0837, .0822, .0807,
    .07919, .07768, .07617, .07465, .07314, .07163, .07, .06838, .06675,
    .06514, .06352, .06191, .0603, .0587, .0571, .05551, .05374, .05199,
    .05024, .0485, .04677, .04504, .04333, .04162, .03992, .03824, .03624,
    .03426, .0323, .03037, .02848, .02662, .02481, .02305, .02134, .0197,
    .01597, .01263, .00968, .00712, .00495, .00317, .00178, 7.9e-4, 2e-4, 0.
  };

  double b[251], c__[251], d__[251];
  int j;
  double xx;
  /* Parameter adjustments */
  --psi;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  spline_(c__251, phild, psild, b, c__, d__);

/* OK */
  for (j = 1; j <= 201; ++j) {
    seval3_(c__251, &phi[j], phild, psild, b, c__, d__, &psi[j], &xx, &xx, &xx);
/* !!      CALL LCSFIT (251, PHILD, PSILD, .TRUE., 'B', 201, PHI, PSI, PSI) */
/* L201: */
  }
  return 0;
}				/* phps65_ */

/* + */
/* ----------------------------------- End of subroutine PHPS65 */
/* Subroutine */ int
phep66_(double *phi, double *eps)
{
  /* Initialized data */

  static double phild[201] = { 0., .0157, .03139, .04709, .06279, .07849,
    .0942, .10991, .12563, .14135, .15708, .17277, .18847, .20417, .21987,
    .23559, .2513, .26701, .28273, .29844, .31416, .32987, .34558, .36129,
    .37699, .3927, .40841, .42411, .43982, .45553, .47124, .48694, .50265,
    .51836, .53406, .54977, .56548, .58119, .5969, .61261, .62832, .64403,
    .65974, .67545, .69116, .70686, .72257, .73828, .75399, .76969, .7854,
    .80111, .81682, .83253, .84824, .86394, .87965, .89536, .91107, .92677,
    .94248, .95819, .9739, .98961, 1.00532, 1.02103, 1.03673, 1.05244,
    1.06815, 1.08385, 1.09956, 1.11527, 1.13098, 1.14669, 1.1624, 1.17811,
    1.19381, 1.20952, 1.22523, 1.24093, 1.25664, 1.27235, 1.28806, 1.30377,
    1.31948, 1.33519, 1.3509, 1.3666, 1.38231, 1.39802, 1.41372, 1.42943,
    1.44514, 1.46085, 1.47656, 1.49227, 1.50798, 1.52368, 1.53939, 1.5551,
    1.5708, 1.58651, 1.60223, 1.61794, 1.63365, 1.64936, 1.66507, 1.68077,
    1.69648, 1.71218, 1.72788, 1.74359, 1.75931, 1.77501, 1.79072, 1.80643,
    1.82214, 1.83784, 1.85355, 1.86925, 1.88496, 1.90066, 1.91636, 1.93206,
    1.94776, 1.96347, 1.97918, 1.99489, 2.0106, 2.02632, 2.04204, 2.05773,
    2.07343, 2.08913, 2.10484, 2.12054, 2.13625, 2.15196, 2.16768, 2.18339,
    2.19911, 2.21482, 2.23052, 2.24623, 2.26194, 2.27765, 2.29336, 2.30906,
    2.32477, 2.34048, 2.35619, 2.37191, 2.38762, 2.40333, 2.41905, 2.43476,
    2.45046, 2.46617, 2.48187, 2.49757, 2.51327, 2.529, 2.54473, 2.56045,
    2.57616, 2.59188, 2.60758, 2.62328, 2.63898, 2.65467, 2.67035, 2.68609,
    2.70183, 2.71756, 2.73328, 2.74899, 2.76469, 2.78039, 2.79608, 2.81176,
    2.82743, 2.84317, 2.85891, 2.87463, 2.89035, 2.90606, 2.92176, 2.93746,
    2.95315, 2.96883, 2.98451, 3.00023, 3.01595, 3.03167, 3.04738, 3.06309,
    3.07879, 3.09449, 3.11019, 3.12589, 3.14159
  };
  static double epsld[201] = { 0., .00145, .0029, .00433, .00574, .00712,
    .00847, .00978, .01105, .01225, .0134, .01447, .01547, .01638, .01719,
    .01789, .01847, .01893, .01924, .0194, .0194, .01924, .01893, .0185,
    .01799, .01741, .01679, .01616, .01556, .01499, .0145, .0141, .01379,
    .01356, .0134, .01331, .01327, .01328, .01333, .0134, .0135, .01361,
    .01373, .01387, .01402, .01419, .01438, .01458, .0148, .01504, .0153,
    .01558, .01588, .0162, .01654, .01689, .01726, .01765, .01805, .01847,
    .0189, .01934, .0198, .02026, .02074, .02124, .02174, .02226, .02279,
    .02334, .0239, .02447, .02506, .02566, .02627, .0269, .02754, .02819,
    .02885, .02952, .0302, .03089, .0316, .03231, .03304, .03378, .03453,
    .0353, .03608, .03688, .0377, .03853, .03938, .04025, .04113, .04202,
    .04293, .04386, .04479, .04574, .0467, .04767, .04866, .04966, .05067,
    .05171, .05277, .05386, .05498, .05612, .0573, .05851, .05976, .06103,
    .06231, .06362, .06493, .06625, .06758, .06889, .0702, .07149, .07277,
    .07402, .07524, .07644, .0776, .07872, .07979, .08082, .0818, .08272,
    .08359, .0844, .08515, .08585, .08649, .08708, .08761, .08808, .0885,
    .08886, .08916, .08941, .08959, .08972, .08978, .08978, .08972, .08959,
    .0894, .08914, .08882, .08843, .08797, .08745, .08687, .08622, .08551,
    .08474, .0839, .083, .08203, .08101, .07991, .07875, .07752, .07622,
    .07485, .07341, .0719, .07031, .06864, .06691, .0651, .06323, .06129,
    .05928, .05722, .05509, .0529, .05064, .04833, .04596, .04354, .04108,
    .03856, .03601, .03341, .03077, .0281, .02539, .02264, .01987, .01707,
    .01426, .01143, .00858, .00573, .00287, 0.
  };

  int j;

  /* Parameter adjustments */
  --eps;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  for (j = 1; j <= 201; ++j) {
/* OK */
    phi[j] = phild[j - 1];
    eps[j] = epsld[j - 1];
/* L201: */
  }
  return 0;
}				/* phep66_ */

/* + */
/* ----------------------------------- End of subroutine PHEP66 */
/* Subroutine */ int
phps66_(double *phi, double *psi)
{
  /* Initialized data */

  static double phild[201] = { 0., .01573, .03145, .04718, .0629, .07862,
    .09433, .11003, .12572, .14141, .15708, .1728, .18851, .20422, .21992,
    .23562, .25131, .26701, .28272, .29844, .31416, .32984, .34552, .36122,
    .37693, .39264, .40835, .42407, .4398, .45552, .47124, .48696, .50267,
    .51838, .53409, .54979, .5655, .58121, .59691, .61262, .62832, .64403,
    .65973, .67544, .69115, .70686, .72257, .73827, .75398, .76969, .7854,
    .80111, .81682, .83252, .84823, .86394, .87965, .89536, .91106, .92677,
    .94248, .95819, .9739, .9896, 1.00531, 1.02102, 1.03673, 1.05244, 1.06814,
    1.08385, 1.09956, 1.11527, 1.13098, 1.14668, 1.16239, 1.1781, 1.19381,
    1.20952, 1.22522, 1.24093, 1.25664, 1.27235, 1.28806, 1.30376, 1.31947,
    1.33518, 1.35089, 1.3666, 1.3823, 1.39801, 1.41372, 1.42943, 1.44514,
    1.46085, 1.47655, 1.49226, 1.50797, 1.52368, 1.53939, 1.55509, 1.5708,
    1.58651, 1.60222, 1.61793, 1.63364, 1.64934, 1.66505, 1.68076, 1.69647,
    1.71217, 1.72788, 1.7436, 1.75932, 1.77504, 1.79075, 1.80647, 1.82217,
    1.83788, 1.85358, 1.86927, 1.88496, 1.9007, 1.91642, 1.93214, 1.94786,
    1.96357, 1.97927, 1.99497, 2.01067, 2.02636, 2.04204, 2.05777, 2.07349,
    2.0892, 2.10492, 2.12062, 2.13633, 2.15203, 2.16773, 2.18342, 2.19911,
    2.21483, 2.23055, 2.24626, 2.26197, 2.27768, 2.29338, 2.30909, 2.32479,
    2.34049, 2.35619, 2.3719, 2.38761, 2.40331, 2.41902, 2.43473, 2.45043,
    2.46614, 2.48185, 2.49756, 2.51327, 2.52897, 2.54467, 2.56037, 2.57607,
    2.59178, 2.60749, 2.6232, 2.63891, 2.65463, 2.67035, 2.68604, 2.70173,
    2.71742, 2.73312, 2.74882, 2.76453, 2.78025, 2.79597, 2.8117, 2.82743,
    2.84311, 2.8588, 2.87449, 2.89019, 2.9059, 2.92161, 2.93733, 2.95305,
    2.96878, 2.98451, 3.0002, 3.0159, 3.0316, 3.04731, 3.06302, 3.07873,
    3.09444, 3.11016, 3.12587, 3.14159
  };
  static double psild[201] = { .16457, .16455, .16449, .16437, .16416,
    .16386, .16345, .16292, .16223, .16139, .16037, .15916, .15779, .15631,
    .15475, .15316, .15157, .15002, .14856, .14722, .14604, .14506, .14427,
    .14364, .14316, .14281, .14257, .14242, .14235, .14233, .14236, .14241,
    .14248, .14257, .14267, .1428, .14294, .1431, .14327, .14346, .14366,
    .14387, .1441, .14433, .14457, .14481, .14506, .1453, .14554, .14578,
    .14601, .14623, .14645, .14665, .14685, .14704, .14722, .1474, .14757,
    .14774, .1479, .14806, .14821, .14835, .14849, .14862, .14875, .14886,
    .14897, .14908, .14917, .14925, .14933, .1494, .14945, .1495, .14954,
    .14957, .14959, .14961, .14961, .1496, .14959, .14956, .14953, .14948,
    .14943, .14936, .14928, .14918, .14908, .14896, .14883, .14869, .14853,
    .14835, .14816, .14796, .14774, .1475, .14725, .14698, .14669, .14638,
    .14606, .14571, .14533, .14494, .14452, .14407, .1436, .1431, .14256,
    .14198, .14135, .14067, .13992, .1391, .1382, .13722, .13615, .13498,
    .13371, .13236, .13093, .12942, .12786, .12623, .12456, .12284, .12108,
    .11929, .11746, .11561, .11373, .11182, .10987, .1079, .1059, .10386,
    .1018, .0997, .09758, .09542, .09325, .09106, .08885, .08662, .08439,
    .08214, .07989, .07763, .07537, .07311, .07085, .06859, .06633, .06407,
    .06182, .05957, .05733, .0551, .05287, .05066, .04846, .04627, .04411,
    .04196, .03983, .03773, .03566, .03362, .03161, .02964, .0277, .0258,
    .02395, .02215, .02039, .01869, .01705, .01547, .01396, .0125, .01112,
    .00981, .00857, .0074, .00631, .00531, .00439, .00356, .00281, .00216,
    .00159, .0011, 7.1e-4, 4e-4, 1.8e-4, 4e-5, 0.
  };

  double b[201], c__[201], d__[201];
  int j;
  double xx;
/*   -------------------------------------------------------------------- */
/*     PURPOSE - */


/* *********************************************************************** */
/*     A R G U M E N T S                                                * */
/* *********************************************************************** */
/* *********************************************************************** */
/*     L O C A L   V A R I A B L E S                                    * */
/* *********************************************************************** */
/* *********************************************************************** */
/*     L O C A L   A R R A Y S                                          * */
/* *********************************************************************** */
  /* Parameter adjustments */
  --psi;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  spline_(c__201, phild, psild, b, c__, d__);

/* OK */
  for (j = 1; j <= 201; ++j) {
    seval3_(c__201, &phi[j], phild, psild, b, c__, d__, &psi[j], &xx, &xx, &xx);
/* !!      CALL LCSFIT (201, PHILD, PSILD, .TRUE., 'B', 201, PHI, PSI, PSI) */
/* L201: */
  }
  return 0;
}				/* phps66_ */

/* + */
/* ----------------------------------- End of subroutine PHPS66 */
/* Subroutine */ int
phep67_(double *phi, double *eps)
{
  /* Initialized data */

  static double phild[251] = { 0., .01495, .0299, .04485, .0598, .07476,
    .08971, .10468, .11964, .13461, .14959, .1528, .15601, .15922, .16243,
    .16565, .16886, .17207, .17529, .1785, .18172, .18687, .19201, .19716,
    .20231, .20747, .21262, .21777, .22293, .22808, .23323, .24276, .25229,
    .26182, .27135, .28088, .29041, .29994, .30947, .319, .32853, .3418,
    .35507, .36834, .38162, .39489, .40817, .42144, .43472, .44799, .46127,
    .47164, .48201, .49238, .50274, .51311, .52348, .53385, .54422, .55458,
    .56495, .57385, .58275, .59165, .60055, .60945, .61835, .62725, .63615,
    .64505, .65395, .66928, .68461, .69994, .71527, .7306, .74593, .76126,
    .77658, .79191, .80724, .82061, .83397, .84734, .8607, .87407, .88743,
    .9008, .91416, .92753, .94089, .95309, .9653, .9775, .9897, 1.0019, 1.0141,
    1.0263, 1.0385, 1.0507, 1.06289, 1.07435, 1.0858, 1.09726, 1.10871,
    1.12017, 1.13162, 1.14308, 1.15453, 1.16599, 1.17744, 1.1884, 1.19936,
    1.21033, 1.22129, 1.23225, 1.24321, 1.25417, 1.26513, 1.27609, 1.28706,
    1.2977, 1.30835, 1.319, 1.32965, 1.3403, 1.35094, 1.36159, 1.37224,
    1.38289, 1.39353, 1.40401, 1.4145, 1.42498, 1.43546, 1.44594, 1.45642,
    1.4669, 1.47738, 1.48786, 1.49834, 1.50878, 1.51921, 1.52965, 1.54008,
    1.55052, 1.56095, 1.57139, 1.58182, 1.59226, 1.60269, 1.61319, 1.62369,
    1.6342, 1.6447, 1.6552, 1.6657, 1.6762, 1.6867, 1.6972, 1.7077, 1.71841,
    1.72913, 1.73984, 1.75056, 1.76127, 1.77199, 1.7827, 1.79341, 1.80413,
    1.81484, 1.82589, 1.83694, 1.84799, 1.85904, 1.87009, 1.88113, 1.89218,
    1.90323, 1.91427, 1.92531, 1.93688, 1.94844, 1.95999, 1.97155, 1.98311,
    1.99467, 2.00622, 2.01778, 2.02933, 2.04089, 2.05305, 2.06522, 2.07739,
    2.08956, 2.10172, 2.11389, 2.12606, 2.13824, 2.15041, 2.16259, 2.17534,
    2.1881, 2.20085, 2.21361, 2.22637, 2.23914, 2.2519, 2.26467, 2.27744,
    2.29021, 2.30376, 2.31733, 2.33089, 2.34445, 2.35802, 2.37159, 2.38516,
    2.39874, 2.41232, 2.4259, 2.44078, 2.45566, 2.47054, 2.48542, 2.50029,
    2.51515, 2.53, 2.54483, 2.55965, 2.57444, 2.59147, 2.60847, 2.62544,
    2.64239, 2.65933, 2.67625, 2.69317, 2.71007, 2.72698, 2.74389, 2.78356,
    2.82326, 2.86298, 2.90274, 2.94251, 2.98231, 3.02211, 3.06193, 3.10176,
    3.14159
  };
  static double epsld[251] = { 0., .00161, .00322, .00482, .0064, .00796,
    .00949, .011, .01246, .01388, .01525, .01554, .01582, .0161, .01637,
    .01664, .0169, .01714, .01738, .01761, .01782, .01813, .01841, .01865,
    .01888, .01908, .01927, .01945, .01962, .01979, .01996, .02029, .02063,
    .02097, .02131, .02164, .02195, .02224, .02249, .02269, .02285, .02298,
    .02301, .02296, .02284, .02267, .02245, .0222, .02194, .02168, .02143,
    .02125, .02108, .02093, .02079, .02067, .02055, .02045, .02035, .02026,
    .02017, .0201, .02004, .01998, .01992, .01987, .01983, .01979, .01975,
    .01972, .0197, .01967, .01966, .01967, .01969, .01973, .01978, .01985,
    .01992, .02001, .0201, .02019, .02029, .02039, .02051, .02062, .02075,
    .02088, .02102, .02117, .02133, .02148, .02164, .0218, .02198, .02216,
    .02234, .02254, .02274, .02295, .02316, .02337, .02359, .02381, .02404,
    .02427, .02451, .02476, .02501, .02527, .02553, .02579, .02606, .02632,
    .0266, .02688, .02716, .02745, .02775, .02805, .02836, .02866, .02897,
    .02928, .0296, .02992, .03025, .03059, .03093, .03128, .03163, .03199,
    .03235, .03271, .03309, .03346, .03385, .03424, .03464, .03505, .03546,
    .03588, .0363, .03673, .03717, .03762, .03807, .03852, .03899, .03945,
    .03993, .04041, .0409, .04139, .0419, .04241, .04293, .04346, .044, .04455,
    .04511, .0457, .04629, .0469, .04752, .04815, .0488, .04945, .05011,
    .05078, .05145, .05216, .05288, .0536, .05434, .05509, .05584, .05661,
    .05739, .05818, .05899, .05985, .06072, .0616, .0625, .0634, .06432,
    .06524, .06617, .06711, .06805, .06905, .07005, .07105, .07204, .07302,
    .07398, .07493, .07586, .07677, .07765, .07854, .07939, .0802, .08097,
    .08171, .0824, .08305, .08365, .08421, .08471, .0852, .08563, .08601,
    .08634, .08662, .08685, .08703, .08716, .08726, .08731, .08731, .08725,
    .0871, .08684, .08646, .08594, .08525, .08439, .08333, .08205, .08029,
    .07826, .07599, .07351, .07088, .06811, .06526, .06235, .05943, .05653,
    .04995, .04366, .03763, .03183, .02623, .02079, .01548, .01026, .00511, 0.
  };

  double b[251], c__[251], d__[251];
  int j;
  double xx;
  /* Parameter adjustments */
  --eps;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  spline_(c__251, phild, epsld, b, c__, d__);

/* OK */
  for (j = 1; j <= 201; ++j) {
    phi[j] = (double) (j - 1) * 3.141592654 / 200.;
    seval3_(c__251, &phi[j], phild, epsld, b, c__, d__, &eps[j], &xx, &xx, &xx);
/* !!  201 CALL LCSFIT (251, PHILD, EPSLD, .TRUE., 'B', 201, PHI, EPS, EPS) */
/* L201: */
  }
  return 0;
}				/* phep67_ */

/* + */
/* ----------------------------------- End of subroutine PHEP67 */
/* Subroutine */ int
phps67_(double *phi, double *psi)
{
  /* Initialized data */

  static double phild[251] = { 0., .01499, .02998, .04497, .05995, .07492,
    .08988, .10483, .11976, .13468, .14959, .1528, .15601, .15923, .16244,
    .16565, .16886, .17208, .17529, .1785, .18172, .18686, .19201, .19716,
    .20231, .20746, .21262, .21777, .22293, .22808, .23323, .24277, .25229,
    .26182, .27135, .28088, .29041, .29993, .30946, .31899, .32853, .34178,
    .35505, .36832, .38159, .39486, .40814, .42142, .4347, .44798, .46127,
    .47164, .482, .49237, .50274, .51311, .52348, .53384, .54421, .55458,
    .56495, .57385, .58275, .59165, .60055, .60945, .61835, .62725, .63615,
    .64505, .65395, .66928, .68461, .69994, .71527, .7306, .74593, .76125,
    .77658, .79191, .80724, .82061, .83397, .84734, .8607, .87407, .88743,
    .9008, .91416, .92753, .94089, .95309, .96529, .97749, .98969, 1.00189,
    1.01409, 1.02629, 1.03849, 1.05069, 1.06289, 1.07435, 1.0858, 1.09726,
    1.10871, 1.12017, 1.13162, 1.14308, 1.15453, 1.16599, 1.17744, 1.1884,
    1.19936, 1.21032, 1.22129, 1.23225, 1.24321, 1.25417, 1.26513, 1.27609,
    1.28706, 1.2977, 1.30835, 1.319, 1.32965, 1.34029, 1.35094, 1.36159,
    1.37224, 1.38289, 1.39353, 1.40401, 1.41449, 1.42498, 1.43546, 1.44594,
    1.45642, 1.4669, 1.47738, 1.48786, 1.49834, 1.50878, 1.51921, 1.52965,
    1.54008, 1.55052, 1.56095, 1.57139, 1.58182, 1.59226, 1.60269, 1.61319,
    1.62369, 1.63419, 1.64469, 1.65519, 1.66569, 1.67619, 1.68669, 1.69719,
    1.7077, 1.71841, 1.72912, 1.73984, 1.75055, 1.76127, 1.77198, 1.7827,
    1.79341, 1.80412, 1.81484, 1.82589, 1.83694, 1.84798, 1.85903, 1.87008,
    1.88113, 1.89218, 1.90322, 1.91427, 1.92531, 1.93688, 1.94844, 1.96,
    1.97156, 1.98312, 1.99468, 2.00623, 2.01779, 2.02934, 2.04089, 2.05308,
    2.06526, 2.07745, 2.08962, 2.1018, 2.11397, 2.12613, 2.13829, 2.15044,
    2.16259, 2.17538, 2.18816, 2.20093, 2.2137, 2.22646, 2.23922, 2.25197,
    2.26472, 2.27746, 2.29021, 2.30379, 2.31737, 2.33095, 2.34453, 2.3581,
    2.37167, 2.38523, 2.39879, 2.41235, 2.4259, 2.44078, 2.45566, 2.47053,
    2.48539, 2.50025, 2.5151, 2.52995, 2.54478, 2.55962, 2.57444, 2.59138,
    2.60831, 2.62523, 2.64216, 2.65909, 2.67603, 2.69297, 2.70993, 2.7269,
    2.74389, 2.78335, 2.8229, 2.86255, 2.90227, 2.94205, 2.98189, 3.02178,
    3.0617, 3.10164, 3.14159
  };
  static double psild[251] = { .18028, .18026, .18018, .18002, .17976,
    .17938, .17885, .17815, .17725, .17614, .1748, .17447, .17414, .1738,
    .17345, .1731, .17276, .17241, .17207, .17174, .17141, .17092, .17045,
    .17001, .16959, .16918, .16879, .1684, .16802, .16764, .16726, .16653,
    .16578, .16502, .16426, .16349, .16272, .16196, .16121, .16048, .15978,
    .15885, .15798, .15717, .15641, .15572, .15507, .15449, .15396, .15348,
    .15306, .15277, .15251, .15228, .15207, .15188, .15171, .15156, .15142,
    .15129, .15116, .15105, .15095, .15085, .15075, .15066, .15057, .15049,
    .15041, .15034, .15027, .15016, .15007, .15, .14994, .1499, .14986, .14984,
    .14982, .14982, .14982, .14982, .14983, .14985, .14986, .14988, .14991,
    .14993, .14996, .14999, .15002, .15005, .15008, .15011, .15014, .15018,
    .15021, .15024, .15028, .15031, .15035, .15038, .15042, .15045, .15048,
    .15052, .15056, .15059, .15063, .15067, .1507, .15074, .15077, .15081,
    .15084, .15088, .15091, .15094, .15096, .15099, .15101, .15103, .15104,
    .15106, .15107, .15108, .15108, .15109, .15109, .1511, .1511, .1511, .1511,
    .1511, .15109, .15109, .15108, .15107, .15106, .15104, .15102, .151,
    .15097, .15094, .1509, .15086, .15081, .15076, .15071, .15065, .15059,
    .15053, .15046, .15038, .1503, .15022, .15013, .15003, .14993, .14982,
    .1497, .14957, .14943, .14929, .14914, .14897, .1488, .14862, .14842,
    .14822, .148, .14776, .14752, .14725, .14698, .14669, .14639, .14607,
    .14574, .14539, .14502, .14462, .1442, .14376, .14329, .14279, .14227,
    .14171, .14111, .14048, .13981, .13905, .13825, .13739, .13648, .13551,
    .13449, .13341, .13227, .13106, .1298, .12839, .12692, .12538, .1238,
    .12216, .12048, .11876, .117, .11523, .11343, .1115, .10955, .10758,
    .10559, .10358, .10154, .09948, .09738, .09526, .0931, .0907, .08825,
    .08576, .08324, .08068, .07808, .07545, .07278, .07008, .06735, .0642,
    .06102, .05782, .05463, .05145, .0483, .04518, .04212, .03912, .0362,
    .02979, .02391, .01859, .01386, .00977, .00635, .00362, .00163, 4.1e-4, 0.
  };

  double b[251], c__[251], d__[251];
  int j;
  double xx;
  /* Parameter adjustments */
  --psi;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  spline_(c__251, phild, psild, b, c__, d__);

/* OK */
  for (j = 1; j <= 201; ++j) {
    seval3_(c__251, &phi[j], phild, psild, b, c__, d__, &psi[j], &xx, &xx, &xx);
/* !!      CALL LCSFIT (251, PHILD, PSILD, .TRUE., 'B', 201, PHI, PSI, PSI) */
/* L201: */
  }
  return 0;
}				/* phps67_ */

/* + */
/* ----------------------------------- End of subroutine PHPS67 */
/* Subroutine */ int
phep63a_(double *phi, double *eps)
{
  /* Initialized data */

  static double phild[251] = { 0., .01459, .02917, .04376, .05835, .07294,
    .08754, .10213, .11673, .13133, .14594, .14925, .15256, .15587, .15919,
    .1625, .16581, .16912, .17244, .17575, .17907, .1842, .18933, .19447,
    .1996, .20474, .20988, .21501, .22015, .22529, .23043, .24002, .2496,
    .25918, .26877, .27835, .28793, .29751, .30709, .31667, .32626, .33981,
    .35335, .36691, .38046, .39401, .40756, .42111, .43466, .44821, .46176,
    .47241, .48306, .49371, .50436, .51501, .52566, .53631, .54696, .55761,
    .56826, .57741, .58655, .5957, .60485, .61399, .62314, .63229, .64143,
    .65058, .65973, .67542, .69111, .7068, .72249, .73818, .75386, .76955,
    .78524, .80093, .81662, .83032, .84402, .85772, .87143, .88513, .89883,
    .91253, .92623, .93993, .95363, .96612, .9786, .99109, 1.00358, 1.01607,
    1.02856, 1.04104, 1.05353, 1.06602, 1.0785, 1.09023, 1.10196, 1.11369,
    1.12541, 1.13714, 1.14887, 1.1606, 1.17232, 1.18405, 1.19577, 1.20699,
    1.21821, 1.22942, 1.24064, 1.25186, 1.26307, 1.27429, 1.28551, 1.29672,
    1.30794, 1.31876, 1.32958, 1.34041, 1.35123, 1.36205, 1.37288, 1.3837,
    1.39453, 1.40535, 1.41617, 1.42675, 1.43732, 1.44789, 1.45847, 1.46904,
    1.47961, 1.49019, 1.50076, 1.51133, 1.52191, 1.53232, 1.54273, 1.55315,
    1.56356, 1.57397, 1.58439, 1.5948, 1.60522, 1.61563, 1.62605, 1.63637,
    1.64669, 1.65701, 1.66733, 1.67765, 1.68797, 1.69829, 1.70862, 1.71894,
    1.72926, 1.73961, 1.74996, 1.76031, 1.77066, 1.78101, 1.79136, 1.80171,
    1.81206, 1.82241, 1.83276, 1.84322, 1.85368, 1.86414, 1.87461, 1.88507,
    1.89553, 1.906, 1.91646, 1.92692, 1.93738, 1.94807, 1.95875, 1.96943,
    1.98011, 1.9908, 2.00148, 2.01216, 2.02284, 2.03353, 2.04421, 2.05529,
    2.06637, 2.07745, 2.08853, 2.09961, 2.11068, 2.12176, 2.13284, 2.14392,
    2.155, 2.16671, 2.17842, 2.19013, 2.20184, 2.21355, 2.22526, 2.23697,
    2.24868, 2.26038, 2.27209, 2.28483, 2.29758, 2.31032, 2.32306, 2.3358,
    2.34855, 2.36129, 2.37403, 2.38677, 2.39951, 2.41402, 2.42853, 2.44304,
    2.45755, 2.47205, 2.48656, 2.50106, 2.51556, 2.53005, 2.54455, 2.56243,
    2.58031, 2.59819, 2.61606, 2.63393, 2.6518, 2.66966, 2.68753, 2.70539,
    2.72325, 2.76507, 2.8069, 2.84873, 2.89056, 2.9324, 2.97423, 3.01607,
    3.05791, 3.09975, 3.14159
  };
  static double epsld[251] = { 0., .00138, .00276, .00413, .00548, .00683,
    .00815, .00945, .01072, .01196, .01317, .01344, .0137, .01396, .01421,
    .01445, .01469, .0149, .0151, .01529, .01546, .01568, .01585, .01599,
    .0161, .01617, .01623, .01628, .01631, .01635, .01638, .01647, .01657,
    .0167, .01684, .01698, .01713, .01728, .01741, .01754, .01764, .01775,
    .01781, .01785, .01787, .01787, .01788, .01788, .01791, .01796, .01804,
    .01814, .01826, .01841, .01857, .01875, .01895, .01917, .01939, .01963,
    .01987, .02009, .02031, .02053, .02075, .02098, .02121, .02144, .02168,
    .02192, .02216, .02257, .02299, .02342, .02385, .02429, .02475, .02521,
    .02569, .02618, .02669, .02715, .02762, .0281, .02859, .02909, .0296,
    .03011, .03063, .03115, .03168, .03216, .03265, .03314, .03363, .03412,
    .03462, .03513, .03564, .03616, .03669, .03719, .03769, .03821, .03873,
    .03925, .03979, .04033, .04087, .04143, .04198, .04253, .04307, .04362,
    .04418, .04473, .04528, .04584, .04639, .04694, .04749, .04801, .04853,
    .04904, .04955, .05005, .05056, .05106, .05155, .05205, .05254, .05302,
    .0535, .05398, .05445, .05492, .05538, .05585, .0563, .05676, .05721,
    .05764, .05807, .0585, .05892, .05933, .05973, .06012, .0605, .06088,
    .06124, .06158, .06192, .06224, .06255, .06286, .06315, .06344, .06372,
    .06399, .06425, .06451, .06476, .065, .06523, .06546, .06567, .06588,
    .06608, .06626, .06643, .0666, .06675, .06689, .06702, .06714, .06724,
    .06733, .0674, .06746, .06751, .06754, .06755, .06756, .06754, .06752,
    .06748, .06742, .06736, .06728, .06718, .06707, .06695, .06681, .06667,
    .0665, .06632, .06613, .06593, .06571, .06548, .06522, .06494, .06464,
    .06434, .06401, .06367, .06332, .06295, .06257, .06217, .06173, .06126,
    .06078, .06029, .05979, .05927, .05875, .05821, .05767, .05712, .05648,
    .05583, .05517, .05448, .05376, .053, .05222, .05138, .0505, .04957,
    .04834, .04703, .04565, .04421, .04273, .0412, .03965, .03807, .03649,
    .03491, .03125, .02764, .02408, .02056, .01708, .01363, .0102, .00679,
    .00339, 0.
  };

  double b[251], c__[251], d__[251];
  int j;
  double xx;
  /* Parameter adjustments */
  --eps;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  spline_(c__251, phild, epsld, b, c__, d__);

/* OK */
  for (j = 1; j <= 201; ++j) {
    phi[j] = (double) (j - 1) * 3.141592654 / 200.;
    seval3_(c__251, &phi[j], phild, epsld, b, c__, d__, &eps[j], &xx, &xx, &xx);
/* !!  201 CALL LCSFIT (251, PHILD, EPSLD, .TRUE., 'B', 201, PHI, EPS, EPS) */
/* L201: */
  }
  return 0;
}				/* phep63a_ */

/* + */
/* -------------------------------- End of Subroutine PHEP63A */
/* Subroutine */ int
phps63a_(double *phi, double *psi)
{
  /* Initialized data */

  static double phild[251] = { 0., .01461, .02922, .04383, .05843, .07303,
    .08763, .10222, .1168, .13138, .14594, .14925, .15257, .15588, .15919,
    .1625, .16582, .16913, .17244, .17576, .17907, .1842, .18933, .19447,
    .1996, .20474, .20987, .21501, .22015, .22529, .23043, .24001, .24958,
    .25916, .26875, .27833, .28791, .2975, .30708, .31667, .32626, .3398,
    .35335, .3669, .38045, .39401, .40756, .42111, .43466, .44821, .46176,
    .47241, .48306, .49371, .50436, .51501, .52566, .53631, .54696, .55761,
    .56826, .57741, .58655, .5957, .60485, .61399, .62314, .63229, .64143,
    .65058, .65973, .67541, .6911, .70679, .72248, .73817, .75386, .76955,
    .78524, .80093, .81662, .83032, .84402, .85772, .87142, .88512, .89882,
    .91253, .92623, .93993, .95363, .96612, .9786, .99109, 1.00358, 1.01607,
    1.02855, 1.04104, 1.05353, 1.06602, 1.0785, 1.09023, 1.10196, 1.11369,
    1.12541, 1.13714, 1.14887, 1.16059, 1.17232, 1.18405, 1.19577, 1.20699,
    1.21821, 1.22943, 1.24064, 1.25186, 1.26308, 1.27429, 1.28551, 1.29672,
    1.30794, 1.31876, 1.32959, 1.34041, 1.35124, 1.36206, 1.37289, 1.38371,
    1.39453, 1.40535, 1.41617, 1.42675, 1.43733, 1.4479, 1.45847, 1.46905,
    1.47962, 1.49019, 1.50077, 1.51134, 1.52191, 1.53232, 1.54274, 1.55316,
    1.56357, 1.57399, 1.5844, 1.59481, 1.60523, 1.61564, 1.62605, 1.63637,
    1.6467, 1.65702, 1.66734, 1.67766, 1.68798, 1.6983, 1.70862, 1.71894,
    1.72926, 1.73961, 1.74996, 1.76031, 1.77067, 1.78101, 1.79136, 1.80171,
    1.81206, 1.82241, 1.83276, 1.84322, 1.85369, 1.86415, 1.87461, 1.88508,
    1.89554, 1.906, 1.91646, 1.92692, 1.93738, 1.94807, 1.95875, 1.96943,
    1.98012, 1.9908, 2.00148, 2.01216, 2.02284, 2.03353, 2.04421, 2.05529,
    2.06637, 2.07744, 2.08852, 2.0996, 2.11068, 2.12176, 2.13284, 2.14392,
    2.155, 2.16671, 2.17841, 2.19012, 2.20183, 2.21354, 2.22525, 2.23696,
    2.24867, 2.26038, 2.27209, 2.28483, 2.29757, 2.31031, 2.32305, 2.33579,
    2.34854, 2.36128, 2.37402, 2.38676, 2.39951, 2.41401, 2.42851, 2.44302,
    2.45752, 2.47203, 2.48653, 2.50103, 2.51554, 2.53004, 2.54455, 2.56241,
    2.58028, 2.59814, 2.61601, 2.63388, 2.65175, 2.66962, 2.6875, 2.70537,
    2.72325, 2.76503, 2.80682, 2.84864, 2.89046, 2.9323, 2.97414, 3.016,
    3.05786, 3.09973, 3.14159
  };
  static double psild[251] = { .15517, .15507, .15478, .15431, .15368, .1529,
    .15199, .15097, .14984, .14863, .14735, .14705, .14675, .14645, .14615,
    .14585, .14555, .14525, .14496, .14468, .1444, .14397, .14357, .14318,
    .14281, .14245, .1421, .14177, .14145, .14115, .14086, .14035, .13987,
    .13944, .13904, .13867, .13833, .13802, .13773, .13746, .13722, .1369,
    .13662, .13637, .13614, .13594, .13577, .13562, .13548, .13537, .13527,
    .1352, .13514, .13509, .13504, .135, .13496, .13492, .13489, .13486,
    .13483, .13481, .13478, .13476, .13474, .13471, .13469, .13467, .13464,
    .13462, .1346, .13456, .13452, .13447, .13442, .13438, .13432, .13427,
    .1342, .13414, .13406, .134, .13392, .13384, .13376, .13367, .13357,
    .13347, .13336, .13324, .13312, .13299, .13287, .13274, .1326, .13245,
    .1323, .13214, .13197, .1318, .13162, .13145, .13127, .13108, .13089,
    .13068, .13047, .13024, .13001, .12976, .1295, .12923, .12896, .12867,
    .12837, .12806, .12774, .12741, .12706, .1267, .12634, .12597, .1256,
    .12521, .12481, .1244, .12398, .12355, .1231, .12264, .12217, .1217,
    .12121, .12072, .12021, .11969, .11916, .11862, .11807, .1175, .11693,
    .11636, .11577, .11518, .11458, .11397, .11335, .11272, .11208, .11143,
    .11078, .11012, .10945, .10878, .1081, .10741, .10671, .10601, .1053,
    .10458, .10385, .10312, .10238, .10164, .10089, .10013, .09936, .0986,
    .09782, .09704, .09625, .09545, .09465, .09384, .09302, .0922, .09137,
    .09054, .08971, .08887, .08802, .08716, .08629, .08541, .08454, .08366,
    .08278, .08189, .081, .08012, .07923, .0783, .07738, .07646, .07553,
    .07461, .07369, .07278, .07186, .07095, .07005, .0691, .06815, .06721,
    .06628, .06535, .06443, .06351, .06259, .06168, .06078, .0598, .05882,
    .05785, .05688, .05593, .05497, .05403, .05309, .05216, .05123, .05019,
    .04916, .04813, .04711, .04608, .04506, .04403, .04299, .04195, .0409,
    .03958, .03826, .03694, .03563, .03435, .03311, .03191, .03077, .0297,
    .02871, .02675, .02524, .02412, .02333, .02282, .02253, .0224, .02238,
    .02241, .02243
  };

  double b[251], c__[251], d__[251];
  int j;
  double xx;

  /* Parameter adjustments */
  --psi;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  spline_(c__251, phild, psild, b, c__, d__);

/* OK */
  for (j = 1; j <= 201; ++j) {
    seval3_(c__251, &phi[j], phild, psild, b, c__, d__, &psi[j], &xx, &xx, &xx);
/* !!      CALL LCSFIT (251, PHILD, PSILD, .TRUE., 'B', 201, PHI, PSI, PSI) */
/* L202: */
  }
  return 0;
}				/* phps63a_ */

/* + */
/* -------------------------------- End of Subroutine PHPS63A */
/* Subroutine */ int
phep64a_(double *phi, double *eps)
{
  /* Initialized data */

  static double phild[251] = { 0., .01468, .02936, .04403, .05872, .0734,
    .08808, .10277, .11746, .13216, .14686, .15019, .15352, .15685, .16018,
    .16351, .16684, .17017, .1735, .17684, .18017, .18528, .19039, .19551,
    .20062, .20574, .21086, .21598, .22109, .22621, .23133, .2409, .25046,
    .26003, .26959, .27915, .28871, .29827, .30783, .31739, .32695, .3404,
    .35384, .36728, .38072, .39417, .40761, .42105, .4345, .44794, .46139,
    .47194, .48249, .49304, .50359, .51414, .52469, .53523, .54578, .55633,
    .56688, .57595, .58502, .5941, .60317, .61224, .62131, .63038, .63945,
    .64852, .65759, .67317, .68874, .70432, .71989, .73546, .75104, .76661,
    .78218, .79776, .81333, .82695, .84057, .85419, .86781, .88142, .89504,
    .90866, .92228, .93589, .94951, .96194, .97437, .98679, .99922, 1.01165,
    1.02407, 1.0365, 1.04892, 1.06135, 1.07377, 1.08546, 1.09714, 1.10882,
    1.1205, 1.13219, 1.14387, 1.15555, 1.16723, 1.17891, 1.19059, 1.2018,
    1.21301, 1.22423, 1.23544, 1.24665, 1.25786, 1.26907, 1.28028, 1.29149,
    1.3027, 1.31357, 1.32444, 1.33531, 1.34618, 1.35705, 1.36792, 1.37878,
    1.38965, 1.40052, 1.41139, 1.42204, 1.43268, 1.44333, 1.45397, 1.46462,
    1.47527, 1.48591, 1.49656, 1.50721, 1.51785, 1.52834, 1.53883, 1.54932,
    1.5598, 1.57029, 1.58078, 1.59127, 1.60176, 1.61225, 1.62274, 1.63312,
    1.64349, 1.65387, 1.66425, 1.67463, 1.68501, 1.69539, 1.70577, 1.71615,
    1.72653, 1.73692, 1.74732, 1.75771, 1.76811, 1.77851, 1.7889, 1.7993,
    1.8097, 1.82009, 1.83049, 1.841, 1.8515, 1.86201, 1.87252, 1.88303,
    1.89354, 1.90405, 1.91455, 1.92506, 1.93557, 1.9463, 1.95704, 1.96777,
    1.9785, 1.98923, 1.99996, 2.01069, 2.02142, 2.03215, 2.04288, 2.05401,
    2.06514, 2.07627, 2.0874, 2.09853, 2.10965, 2.12078, 2.13191, 2.14304,
    2.15416, 2.16592, 2.17767, 2.18943, 2.20118, 2.21293, 2.22469, 2.23644,
    2.24819, 2.25994, 2.27169, 2.28447, 2.29725, 2.31002, 2.32279, 2.33557,
    2.34834, 2.36111, 2.37389, 2.38666, 2.39943, 2.41397, 2.4285, 2.44304,
    2.45757, 2.4721, 2.48663, 2.50115, 2.51568, 2.5302, 2.54471, 2.56262,
    2.58052, 2.59842, 2.61631, 2.6342, 2.65209, 2.66997, 2.68785, 2.70573,
    2.72362, 2.76541, 2.8072, 2.84899, 2.89079, 2.93259, 2.97439, 3.01619,
    3.05799, 3.09979, 3.14159
  };
  static double epsld[251] = { 0., .00149, .00297, .00445, .00591, .00736,
    .00878, .01018, .01156, .0129, .0142, .01449, .01477, .01505, .01532,
    .01558, .01583, .01606, .01628, .01648, .01667, .0169, .01709, .01724,
    .01736, .01745, .01752, .01758, .01762, .01767, .01772, .01782, .01796,
    .01811, .01828, .01845, .01861, .01877, .01891, .01903, .01912, .01919,
    .0192, .01916, .01908, .01897, .01886, .01874, .01862, .01854, .01848,
    .01846, .01847, .01851, .01856, .01864, .01873, .01883, .01895, .01907,
    .01921, .01933, .01945, .01958, .01972, .01985, .01999, .02013, .02028,
    .02043, .02058, .02084, .02111, .02139, .02169, .02199, .02231, .02264,
    .02299, .02335, .02373, .02408, .02444, .02482, .0252, .0256, .02601,
    .02643, .02685, .02728, .02772, .02812, .02852, .02893, .02935, .02977,
    .0302, .03064, .03108, .03153, .03198, .03242, .03286, .03332, .03378,
    .03425, .03472, .03521, .03571, .03622, .03674, .03724, .03776, .03828,
    .03882, .03936, .0399, .04045, .041, .04156, .04212, .04266, .0432, .04375,
    .04429, .04484, .04539, .04594, .04649, .04704, .0476, .04814, .04869,
    .04923, .04978, .05032, .05087, .05141, .05194, .05248, .05301, .05352,
    .05403, .05454, .05504, .05552, .056, .05647, .05693, .05738, .05781,
    .05822, .05862, .05901, .05939, .05976, .06011, .06045, .06079, .06111,
    .06143, .06174, .06204, .06233, .06262, .06289, .06316, .06341, .06365,
    .06389, .06411, .06432, .06452, .06471, .06489, .06505, .0652, .06534,
    .06546, .06557, .06567, .06575, .06582, .06587, .06591, .06594, .06595,
    .06594, .06593, .0659, .06586, .0658, .06573, .06565, .06555, .06544,
    .06531, .06517, .06502, .06485, .06466, .06445, .06422, .06398, .06371,
    .06344, .06314, .06283, .06251, .06216, .0618, .0614, .06097, .06053,
    .06007, .0596, .05912, .05862, .05811, .0576, .05707, .05646, .05584,
    .0552, .05454, .05384, .05312, .05235, .05154, .05068, .04977, .04856,
    .04728, .04592, .04451, .04304, .04154, .04, .03844, .03687, .0353, .03165,
    .02804, .02446, .02091, .01739, .01389, .0104, .00693, .00346, 0.
  };

  double b[251], c__[251], d__[251];
  int j;
  double xx;

  /* Parameter adjustments */
  --eps;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  spline_(c__251, phild, epsld, b, c__, d__);

/* OK */
  for (j = 1; j <= 201; ++j) {
    phi[j] = (double) (j - 1) * 3.141592654 / 200.;
    seval3_(c__251, &phi[j], phild, epsld, b, c__, d__, &eps[j], &xx, &xx, &xx);
/* !!  201 CALL LCSFIT (251, PHILD, EPSLD, .TRUE., 'B', 201, PHI, EPS, EPS) */
/* L201: */
  }
  return 0;
}				/* phep64a_ */

/* + */
/* -------------------------------- End of Subroutine PHEP64A */
/* Subroutine */ int
phps64a_(double *phi, double *psi)
{
  /* Initialized data */

  static double phild[251] = { 0., .0147, .0294, .0441, .0588, .07349, .08818,
    .10286, .11753, .1322, .14686, .15019, .15352, .15685, .16019, .16352,
    .16685, .17018, .17351, .17684, .18017, .18528, .1904, .19551, .20063,
    .20574, .21086, .21598, .22109, .22621, .23133, .24088, .25044, .26,
    .26956, .27912, .28869, .29825, .30782, .31739, .32695, .34039, .35384,
    .36728, .38072, .39416, .40761, .42105, .4345, .44794, .46139, .47194,
    .48249, .49303, .50358, .51413, .52468, .53523, .54578, .55633, .56688,
    .57595, .58502, .59409, .60317, .61224, .62131, .63038, .63945, .64852,
    .65759, .67316, .68874, .70431, .71989, .73546, .75103, .76661, .78218,
    .79776, .81333, .82695, .84057, .85418, .8678, .88142, .89504, .90866,
    .92228, .93589, .94951, .96194, .97436, .98679, .99922, 1.01164, 1.02407,
    1.0365, 1.04892, 1.06135, 1.07377, 1.08546, 1.09714, 1.10882, 1.1205,
    1.13218, 1.14386, 1.15554, 1.16723, 1.17891, 1.19059, 1.2018, 1.21301,
    1.22422, 1.23543, 1.24665, 1.25786, 1.26907, 1.28028, 1.29149, 1.3027,
    1.31357, 1.32444, 1.33531, 1.34618, 1.35705, 1.36792, 1.37879, 1.38966,
    1.40052, 1.41139, 1.42204, 1.43269, 1.44334, 1.45398, 1.46463, 1.47528,
    1.48592, 1.49657, 1.50721, 1.51785, 1.52835, 1.53884, 1.54933, 1.55982,
    1.5703, 1.58079, 1.59128, 1.60177, 1.61225, 1.62274, 1.63312, 1.6435,
    1.65388, 1.66426, 1.67464, 1.68502, 1.6954, 1.70578, 1.71615, 1.72653,
    1.73693, 1.74732, 1.75772, 1.76812, 1.77851, 1.78891, 1.79931, 1.8097,
    1.82009, 1.83049, 1.841, 1.85151, 1.86202, 1.87253, 1.88304, 1.89354,
    1.90405, 1.91456, 1.92507, 1.93557, 1.94631, 1.95704, 1.96777, 1.9785,
    1.98923, 1.99996, 2.01069, 2.02142, 2.03215, 2.04288, 2.05401, 2.06514,
    2.07627, 2.0874, 2.09852, 2.10965, 2.12078, 2.13191, 2.14304, 2.15416,
    2.16592, 2.17767, 2.18942, 2.20117, 2.21293, 2.22468, 2.23643, 2.24819,
    2.25994, 2.27169, 2.28447, 2.29724, 2.31001, 2.32278, 2.33556, 2.34833,
    2.36111, 2.37388, 2.38666, 2.39943, 2.41396, 2.42849, 2.44302, 2.45754,
    2.47207, 2.4866, 2.50113, 2.51566, 2.53019, 2.54471, 2.5626, 2.58049,
    2.59837, 2.61626, 2.63415, 2.65203, 2.66993, 2.68782, 2.70572, 2.72362,
    2.76535, 2.80711, 2.84888, 2.89067, 2.93247, 2.97428, 3.0161, 3.05793,
    3.09976, 3.14159
  };
  static double psild[251] = { .15365, .15355, .15324, .15275, .15209,
    .15129, .15035, .1493, .14816, .14693, .14565, .14535, .14505, .14475,
    .14444, .14414, .14384, .14353, .14323, .14293, .14263, .14218, .14174,
    .1413, .14087, .14044, .14003, .13963, .13924, .13886, .13849, .13783,
    .13722, .13665, .13612, .13562, .13516, .13473, .13432, .13395, .13359,
    .13313, .13271, .13233, .13199, .13167, .13139, .13113, .1309, .13069,
    .13051, .13037, .13024, .13013, .13003, .12993, .12985, .12977, .12971,
    .12966, .12961, .12958, .12955, .12953, .12952, .12951, .1295, .12949,
    .12949, .12949, .12949, .12948, .12947, .12947, .12946, .12945, .12944,
    .12943, .12942, .12941, .1294, .12939, .12939, .12938, .12937, .12936,
    .12934, .12932, .1293, .12927, .12923, .1292, .12915, .1291, .12905,
    .12899, .12892, .12885, .12878, .12869, .12861, .12852, .12843, .12833,
    .12822, .12811, .128, .12788, .12775, .12761, .12747, .12733, .12718,
    .12702, .12685, .12668, .1265, .12631, .12611, .1259, .12569, .12547,
    .12524, .125, .12475, .12448, .1242, .1239, .12358, .12325, .12289, .12252,
    .12212, .12171, .12128, .12083, .12036, .11988, .11938, .11887, .11834,
    .11781, .11727, .11672, .11615, .11557, .11499, .11439, .11378, .11316,
    .11253, .11189, .11125, .11059, .10993, .10925, .10857, .10788, .10719,
    .10648, .10577, .10505, .10433, .1036, .10286, .10212, .10137, .10061,
    .09985, .09908, .09831, .09752, .09672, .09592, .09512, .09431, .09349,
    .09267, .09184, .09101, .09017, .08931, .08845, .08758, .0867, .08583,
    .08495, .08406, .08318, .08229, .0814, .08048, .07955, .07863, .0777,
    .07678, .07585, .07493, .07401, .07309, .07217, .0712, .07023, .06927,
    .06831, .06735, .06639, .06545, .0645, .06356, .06262, .06161, .0606,
    .0596, .05861, .05762, .05664, .05567, .0547, .05375, .05279, .05172,
    .05066, .0496, .04855, .04749, .04644, .04538, .04432, .04324, .04216,
    .04081, .03945, .03809, .03675, .03543, .03414, .03291, .03173, .03061,
    .02958, .02751, .02589, .02465, .02374, .02312, .02273, .02252, .02243,
    .02242, .02243
  };

  double b[251], c__[251], d__[251];
  int j;
  double xx;

  /* Parameter adjustments */
  --psi;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  spline_(c__251, phild, psild, b, c__, d__);

/* OK */
  for (j = 1; j <= 201; ++j) {
    seval3_(c__251, &phi[j], phild, psild, b, c__, d__, &psi[j], &xx, &xx, &xx);
/* !!      CALL LCSFIT (251, PHILD, PSILD, .TRUE., 'B', 201, PHI, PSI, PSI) */
/* L202: */
  }
  return 0;
}				/* phps64a_ */

/* + */
/* -------------------------------- End of Subroutine PHPS64A */
/* Subroutine */ int
phep65a_(double *phi, double *eps)
{
  /* Initialized data */

  static double phild[251] = { 0., .01453, .02907, .0436, .05814, .07267,
    .08721, .10175, .1163, .13084, .14539, .14877, .15214, .15552, .15889,
    .16227, .16564, .16902, .1724, .17578, .17916, .18436, .18956, .19477,
    .19997, .20518, .21039, .21559, .2208, .22601, .23122, .24077, .25032,
    .25987, .26941, .27896, .28851, .29805, .3076, .31714, .32669, .33996,
    .35322, .36649, .37976, .39302, .40629, .41956, .43283, .4461, .45937,
    .46976, .48015, .49054, .50093, .51132, .52171, .5321, .54249, .55288,
    .56327, .57223, .58118, .59014, .59909, .60805, .617, .62596, .63491,
    .64387, .65282, .66829, .68376, .69923, .7147, .73017, .74564, .7611,
    .77657, .79204, .8075, .82106, .83461, .84816, .86172, .87527, .88882,
    .90237, .91593, .92948, .94303, .95538, .96772, .98007, .99242, 1.00476,
    1.01711, 1.02945, 1.0418, 1.05415, 1.06649, 1.0781, 1.08971, 1.10132,
    1.11293, 1.12454, 1.13615, 1.14776, 1.15937, 1.17098, 1.18259, 1.19373,
    1.20486, 1.216, 1.22713, 1.23827, 1.2494, 1.26054, 1.27167, 1.28281,
    1.29394, 1.30473, 1.31552, 1.32631, 1.33711, 1.3479, 1.35869, 1.36948,
    1.38027, 1.39106, 1.40185, 1.41248, 1.42312, 1.43375, 1.44438, 1.45501,
    1.46564, 1.47628, 1.48691, 1.49754, 1.50817, 1.51872, 1.52927, 1.53983,
    1.55038, 1.56093, 1.57148, 1.58203, 1.59258, 1.60314, 1.61369, 1.62418,
    1.63467, 1.64516, 1.65565, 1.66614, 1.67663, 1.68712, 1.69762, 1.70811,
    1.7186, 1.72914, 1.73967, 1.7502, 1.76074, 1.77127, 1.78181, 1.79234,
    1.80288, 1.81341, 1.82395, 1.8346, 1.84526, 1.85591, 1.86657, 1.87722,
    1.88788, 1.89853, 1.90919, 1.91985, 1.9305, 1.94138, 1.95225, 1.96313,
    1.97401, 1.98488, 1.99576, 2.00664, 2.01751, 2.02839, 2.03927, 2.04786,
    2.05645, 2.06504, 2.07363, 2.08222, 2.09081, 2.0994, 2.10799, 2.11658,
    2.12517, 2.13979, 2.15441, 2.16903, 2.18365, 2.19826, 2.21288, 2.2275,
    2.24211, 2.25673, 2.27134, 2.28424, 2.29714, 2.31003, 2.32292, 2.33582,
    2.34871, 2.3616, 2.37449, 2.38739, 2.40028, 2.4149, 2.42952, 2.44413,
    2.45875, 2.47336, 2.48798, 2.50259, 2.51719, 2.5318, 2.5464, 2.56435,
    2.58229, 2.60023, 2.61816, 2.6361, 2.65402, 2.67195, 2.68987, 2.70779,
    2.72571, 2.76731, 2.80891, 2.8505, 2.89209, 2.93368, 2.97526, 3.01684,
    3.05843, 3.10001, 3.14159
  };
  static double epsld[251] = { 0., .00137, .00274, .0041, .00545, .00679,
    .00811, .00941, .0107, .01195, .01318, .01346, .01374, .01401, .01427,
    .01453, .01477, .015, .01521, .01541, .01559, .01583, .01602, .01618,
    .0163, .0164, .01648, .01654, .0166, .01665, .01671, .01683, .01698,
    .01715, .01732, .01749, .01766, .01782, .01796, .01807, .01814, .01819,
    .01816, .01807, .01793, .01775, .01755, .01732, .01708, .01684, .01661,
    .01644, .01628, .01614, .01601, .01589, .01578, .01569, .01561, .01554,
    .01549, .01546, .01543, .01541, .01541, .01541, .01542, .01544, .01547,
    .01551, .01555, .01565, .01577, .0159, .01607, .01625, .01645, .01668,
    .01693, .0172, .0175, .01778, .01807, .01837, .01869, .01902, .01936,
    .0197, .02005, .0204, .02076, .02108, .02141, .02174, .02207, .0224,
    .02274, .02309, .02344, .02381, .02418, .02454, .02491, .02528, .02567,
    .02607, .02647, .02689, .02731, .02775, .02819, .02863, .02907, .02952,
    .02998, .03044, .03091, .03138, .03185, .03232, .0328, .03326, .03371,
    .03417, .03463, .0351, .03557, .03604, .03652, .03701, .0375, .038, .03851,
    .03902, .03954, .04007, .0406, .04114, .04169, .04224, .04279, .04335,
    .04391, .04447, .04502, .04558, .04614, .04669, .04723, .04777, .0483,
    .04882, .04933, .04984, .05033, .05082, .0513, .05177, .05224, .0527,
    .05316, .05361, .05406, .05449, .05493, .05535, .05577, .05618, .05657,
    .05696, .05734, .05771, .05807, .05842, .05876, .05908, .05939, .05969,
    .05997, .06023, .06049, .06073, .06095, .06116, .06135, .06153, .0617,
    .06185, .06199, .06212, .06224, .06232, .0624, .06247, .06254, .06259,
    .06264, .06268, .06272, .06274, .06276, .06277, .06275, .06271, .06264,
    .06255, .06242, .06226, .06207, .06185, .06159, .06134, .06106, .06075,
    .06042, .06007, .0597, .05932, .05891, .0585, .05807, .05756, .05704,
    .0565, .05592, .05531, .05466, .05397, .05323, .05244, .05159, .05046,
    .04925, .04795, .04659, .04517, .0437, .0422, .04065, .03909, .03751,
    .03383, .03013, .0264, .02266, .01891, .01514, .01136, .00758, .00379, 0.
  }
  ;

  double b[251], c__[251], d__[251];
  int j;
  double xx;

  /* Parameter adjustments */
  --eps;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  spline_(c__251, phild, epsld, b, c__, d__);

/* OK */
  for (j = 1; j <= 201; ++j) {
    phi[j] = (double) (j - 1) * 3.141592654 / 200.;
    seval3_(c__251, &phi[j], phild, epsld, b, c__, d__, &eps[j], &xx, &xx, &xx);
/* !!  201 CALL LCSFIT (251, PHILD, EPSLD, .TRUE., 'B', 201, PHI, EPS, EPS) */
/* L201: */
  }
  return 0;
}				/* phep65a_ */

/* + */
/* -------------------------------- End of Subroutine PHEP65A */
/* Subroutine */ int
phps65a_(double *phi, double *psi)
{
  /* Initialized data */

  static double phild[251] = { 0., .01455, .0291, .04364, .05819, .07273,
    .08727, .10181, .11634, .13087, .14539, .14877, .15214, .15552, .1589,
    .16227, .16565, .16903, .1724, .17578, .17916, .18436, .18956, .19477,
    .19997, .20518, .21039, .2156, .2208, .22601, .23122, .24077, .25032,
    .25986, .26941, .27896, .2885, .29805, .3076, .31714, .32669, .33995,
    .35322, .36648, .37974, .39301, .40628, .41955, .43282, .44609, .45937,
    .46975, .48014, .49053, .50092, .51131, .5217, .5321, .54249, .55288,
    .56327, .57223, .58118, .59014, .59909, .60805, .617, .62596, .63491,
    .64387, .65282, .66829, .68376, .69923, .7147, .73016, .74563, .7611,
    .77657, .79204, .8075, .82106, .83461, .84816, .86171, .87527, .88882,
    .90237, .91592, .92948, .94303, .95538, .96772, .98007, .99241, 1.00476,
    1.01711, 1.02945, 1.0418, 1.05414, 1.06649, 1.0781, 1.08971, 1.10132,
    1.11293, 1.12454, 1.13615, 1.14776, 1.15937, 1.17098, 1.18259, 1.19372,
    1.20486, 1.21599, 1.22713, 1.23827, 1.2494, 1.26054, 1.27167, 1.28281,
    1.29394, 1.30473, 1.31552, 1.32631, 1.3371, 1.3479, 1.35869, 1.36948,
    1.38027, 1.39106, 1.40185, 1.41248, 1.42311, 1.43375, 1.44438, 1.45501,
    1.46564, 1.47628, 1.48691, 1.49754, 1.50817, 1.51872, 1.52928, 1.53983,
    1.55038, 1.56094, 1.57149, 1.58204, 1.59259, 1.60314, 1.61369, 1.62418,
    1.63468, 1.64517, 1.65566, 1.66616, 1.67665, 1.68714, 1.69763, 1.70811,
    1.7186, 1.72914, 1.73968, 1.75021, 1.76075, 1.77128, 1.78182, 1.79235,
    1.80289, 1.81342, 1.82395, 1.83461, 1.84527, 1.85592, 1.86658, 1.87723,
    1.88789, 1.89854, 1.9092, 1.91985, 1.9305, 1.94138, 1.95226, 1.96314,
    1.97402, 1.98489, 1.99577, 2.00664, 2.01752, 2.02839, 2.03927, 2.05056,
    2.06185, 2.07314, 2.08444, 2.09573, 2.10702, 2.11831, 2.1296, 2.14089,
    2.15217, 2.16409, 2.17601, 2.18793, 2.19985, 2.21176, 2.22368, 2.2356,
    2.24751, 2.25943, 2.27134, 2.28424, 2.29713, 2.31002, 2.32292, 2.33581,
    2.3487, 2.36159, 2.37449, 2.38738, 2.40028, 2.41489, 2.4295, 2.44411,
    2.45872, 2.47334, 2.48795, 2.50256, 2.51717, 2.53178, 2.5464, 2.56432,
    2.58225, 2.60018, 2.6181, 2.63603, 2.65396, 2.67189, 2.68983, 2.70777,
    2.72571, 2.76723, 2.80877, 2.85033, 2.8919, 2.9335, 2.9751, 3.01671,
    3.05834, 3.09996, 3.14159
  };
  static double psild[251] = { .14761, .14748, .14709, .14649, .1457, .14476,
    .14369, .14253, .14131, .14007, .13882, .13854, .13826, .13798, .1377,
    .13743, .13716, .1369, .13665, .1364, .13616, .13581, .13547, .13515,
    .13484, .13455, .13426, .13398, .13371, .13345, .13318, .1327, .13221,
    .13172, .13124, .13075, .13026, .12976, .12926, .12876, .12825, .12753,
    .12681, .1261, .12541, .12475, .12415, .12359, .12311, .1227, .12239,
    .12221, .12209, .12202, .12199, .12199, .12203, .12209, .12216, .12224,
    .12233, .1224, .12246, .12253, .12259, .12265, .12271, .12277, .12283,
    .12289, .12295, .12305, .12316, .12326, .12337, .12348, .12359, .1237,
    .12381, .12391, .12401, .1241, .12418, .12426, .12434, .12441, .12448,
    .12455, .12461, .12467, .12472, .12477, .12482, .12486, .12489, .12492,
    .12495, .12498, .125, .12501, .12502, .12503, .12503, .12502, .12501, .125,
    .12498, .12495, .12492, .12489, .12485, .12481, .12476, .12471, .12465,
    .12459, .12452, .12445, .12437, .12428, .12419, .12409, .12398, .12387,
    .12375, .12362, .12349, .12334, .12319, .12303, .12286, .12268, .12249,
    .12229, .12208, .12186, .12163, .12138, .12113, .12085, .12057, .12027,
    .11996, .11964, .1193, .11894, .11857, .11818, .11778, .11736, .11693,
    .11648, .11602, .11554, .11504, .11453, .114, .11346, .11291, .11234,
    .11175, .11115, .11054, .10991, .10928, .10863, .10797, .1073, .10662,
    .10594, .10524, .10453, .10381, .10308, .10235, .10161, .10086, .1001,
    .09933, .09856, .09778, .09698, .09617, .09535, .09452, .09369, .09285,
    .09201, .09116, .0903, .08943, .08853, .08763, .08671, .08579, .08487,
    .08394, .08301, .08207, .08113, .08019, .07919, .07818, .07717, .07616,
    .07515, .07413, .07311, .07209, .07106, .07004, .06892, .06781, .0667,
    .06559, .06448, .06338, .06229, .06119, .06011, .05904, .05783, .05664,
    .05545, .05427, .05309, .05191, .05072, .04953, .04834, .04713, .04564,
    .04413, .04263, .04114, .03967, .03823, .03683, .03549, .03421, .033,
    .03052, .02844, .02675, .02539, .02434, .02355, .02301, .02266, .02248,
    .02242
  };

  double b[251], c__[251], d__[251];
  int j;
  double xx;
  /* Parameter adjustments */
  --psi;
  --phi;

  /* Function Body */
/* ----------------------------------------------------------------------- */
  spline_(c__251, phild, psild, b, c__, d__);

/* OK */
  for (j = 1; j <= 201; ++j) {
    seval3_(c__251, &phi[j], phild, psild, b, c__, d__, &psi[j], &xx, &xx, &xx);
/* !!      CALL LCSFIT (251, PHILD, PSILD, .TRUE., 'B', 201, PHI, PSI, PSI) */
/* L202: */
  }
  return 0;
}				/* phps65a_ */

/* + */
/* -------------------------------- End of Subroutine PHPS65A */
/* Subroutine */ int
spline_(int n, double *x, double *y, double *b, double *c__, double *d__)
{
  /* System generated locals */
  int i__1;

  /* Local variables */
  int i__;
  double t;

/*   -------------------------------------------------------------------- */
/*     PURPOSE - Compute the coefficients of a cubic spline */

/*     AUTHORS - Forsythe, Malcolm, & Moler */
/*             - Ralph L. Carmichael, NASA/AMES */

/*     REVISION HISTORY */
/*   DATE  VERS PERSON  STATEMENT OF CHANGES */
/*   1977   1.0   FMM   Original publication */
/*   1981   1.1   RLC   Fortran 77 constructions; IMPLICIT NONE */
/*  9Jul89  1.2   RLC   Added SEVAL3 */
/* 17Apr91  1.3   RLC   Fortran 90 style loops */

/*     NOTES - From "Computer Methods for Mathematical Computations", by */
/*       Forsythe, Malcolm, and Moser. Prentice-Hall, 1977. */

/* *********************************************************************** */
/*     A R G U M E N T S                                                * */
/* *********************************************************************** */
/* # of data points (n>1) */
/* fabscissas of knots */
/* ordinates of knots */
/* linear coeff */
/* quadratic coeff. */
/* *********************************************************************** */
/*     L O C A L   V A R I A B L E S                                    * */
/* *********************************************************************** */
/* cubic coeff. */
/* ----------------------------------------------------------------------- */
  /* Parameter adjustments */
  --d__;
  --c__;
  --b;
  --y;
  --x;

  /* Function Body */
  if (n < 3) {
/* special treatment for n < 3 */
    b[1] = 0.;
    if (n == 2) {
      b[1] = (y[2] - y[1]) / (x[2] - x[1]);
    }
    c__[1] = 0.;
    d__[1] = 0.;
    b[2] = b[1];
    c__[2] = 0.;
    d__[2] = 0.;
    return 0;
/* note PREMATURE return */
  }


/* .....Set up tridiagonal system......................................... */
/* .    b=diagonal, d=offdiagonal, c=right-hand side */
  d__[1] = x[2] - x[1];
  c__[2] = (y[2] - y[1]) / d__[1];
  i__1 = n - 1;
  for (i__ = 2; i__ <= i__1; ++i__) {
    d__[i__] = x[i__ + 1] - x[i__];
    b[i__] = (d__[i__ - 1] + d__[i__]) * 2.;
    c__[i__ + 1] = (y[i__ + 1] - y[i__]) / d__[i__];
    c__[i__] = c__[i__ + 1] - c__[i__];
  }

/* .....End conditions.  third derivatives at x(1) and x(n) obtained */
/* .       from divided differences....................................... */
  b[1] = -d__[1];
  b[n] = -d__[n - 1];
  c__[1] = 0.;
  c__[n] = 0.;
  if (n > 3) {
    c__[1] = c__[3] / (x[4] - x[2]) - c__[2] / (x[3] - x[1]);
    c__[n] = c__[n - 1] / (x[n] - x[n - 2]) - c__[n - 2] / (x[n - 1]
							    - x[n - 3]);
    c__[1] = c__[1] * d__[1] * d__[1] / (x[4] - x[1]);
    c__[n] = -c__[n] * d__[n - 1] * d__[n - 1] / (x[n] - x[n - 3]);
  }

  i__1 = n;
  for (i__ = 2; i__ <= i__1; ++i__) {
/* forward elimination */
    t = d__[i__ - 1] / b[i__ - 1];
    b[i__] -= t * d__[i__ - 1];
    c__[i__] -= t * c__[i__ - 1];
  }

  c__[n] /= b[n];
/* back substitution ( makes C the sigma of text) */
  for (i__ = n - 1; i__ >= 1; --i__) {
    c__[i__] = (c__[i__] - d__[i__] * c__[i__ + 1]) / b[i__];
  }


/* .....Compute polynomial coefficients................................... */
  b[n] = (y[n] - y[n - 1]) / d__[n - 1] + d__[n - 1] * (c__[n - 1] + c__[n] + c__[n]);
  i__1 = n - 1;
  for (i__ = 1; i__ <= i__1; ++i__) {
    b[i__] = (y[i__ + 1] - y[i__]) / d__[i__] - d__[i__] * (c__[i__ + 1]
							    + c__[i__] + c__[i__]);
    d__[i__] = (c__[i__ + 1] - c__[i__]) / d__[i__];
    c__[i__] *= 3.;
  }
  c__[n] *= 3.;
  d__[n] = d__[n - 1];
  return 0;
}				/* spline_ */

/* + */
/* --------------------------------- End of Subroutine SPLINE */

int
seval3_(int n, double *u, double *x,
	double *y, double *b, double *c__, double *d__,
	double *seval, double *deriv, double *deriv2, double *deriv3)
{
  int i__, j, k;
  double dx;

/*   -------------------------------------------------------------------- */
/*     PURPOSE - Evaluate the cubic spline function and its first, */
/*        second and third derivatives. */
/*        seval=y(i)+b(i)*(u-x(i))+c(i)*(u-x(i))**2+d(i)*(u-x(i))**3 */
/*           where  x(i-1) <= u < x(i) */

/*     AUTHORS - Forsythe, Malcolm, and Moser */
/*             - Ralph L. Carmichael, NASA/AMES */

/*     REVISION HISTORY */
/*   DATE  VERS PERSON  STATEMENT OF CHANGES */
/*   1977   1.0   FMM   Original publication */
/*   1981   1.1   RLC   Fortran 77 constructions; IMPLICIT NONE */
/* 16Nov94  1.2   RLC   Added the DERIV3; */

/*     NOTES- if u<x(1), i=1 is used;if u>x(n), i=n is used */
/*       From "Computer Methods for Mathematical Computations", by */
/*                    Forsythe, Malcolm, and Moser. Prentice-Hall, 1977. */

/* *********************************************************************** */
/*     A R G U M E N T S                                                * */
/* *********************************************************************** */
/* # of data points (n>1) */
/* fabscissa at which the spline is to be evaluat */
/* fabscissas of knots */
/* ordinates of knots */
/* linear coeff */
/* quadratic coeff. */
/* cubic coeff. */
/* *********************************************************************** */
/*     L O C A L   V A R I A B L E S                                    * */
/* *********************************************************************** */
/* ----------------------------------------------------------------------- */
  /* Parameter adjustments */
  --d__;
  --c__;
  --b;
  --y;
  --x;

  /* Function Body */
  if (n <= 1) {
    *seval = y[1];
    *deriv = 0.;
    *deriv2 = 0.;
    *deriv3 = 0.;
    return 0;
/* notice the premature return */
  }

  i__ = 1;
/* binary search */
  j = n;
L20:
  k = (i__ + j) / 2;
  if (*u < x[k]) {
    j = k;
  } else {
    i__ = k;
  }
  if (j > i__ + 1) {
    goto L20;
  }

/* .....I has been determined. Evaluate spline and derivatives............ */
  dx = *u - x[i__];
  *seval = y[i__] + dx * (b[i__] + dx * (c__[i__] + dx * d__[i__]));
  *deriv = b[i__] + dx * (c__[i__] + c__[i__] + dx * (d__[i__] + d__[i__] + d__[i__]));
  *deriv2 = (c__[i__] + dx * 3. * d__[i__]) * 2.;
  *deriv3 = d__[i__] * 6.;
  return 0;
}				/* seval3_ */

#ifdef __cplusplus
}
#endif
